@@ -26,13 +26,14 @@
 #import <Foundation/NSGeometry.h>
 
 #import "Platform.h"
-#import "ScintillaView.h"
-#import "ScintillaCocoa.h"
-#import "PlatCocoa.h"
 
 #include "StringCopy.h"
 #include "XPM.h"
 
+#import "ScintillaView.h"
+#import "ScintillaCocoa.h"
+#import "PlatCocoa.h"
+
 using namespace Scintilla;
 
 extern sptr_t scintilla_send_message(void* sci, unsigned int iMessage, uptr_t wParam, sptr_t lParam);
@@ -480,14 +481,14 @@ void SurfaceImpl::FillRectangle(PRectangle rc, ColourDesired back)
 
 //--------------------------------------------------------------------------------------------------
 
-void drawImageRefCallback(CGImageRef pattern, CGContextRef gc)
+static void drawImageRefCallback(CGImageRef pattern, CGContextRef gc)
 {
   CGContextDrawImage(gc, CGRectMake(0, 0, CGImageGetWidth(pattern), CGImageGetHeight(pattern)), pattern);
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void releaseImageRefCallback(CGImageRef pattern)
+static void releaseImageRefCallback(CGImageRef pattern)
 {
   CGImageRelease(pattern);
 }
@@ -1118,7 +1119,7 @@ bool Window::HasFocus()
 
 //--------------------------------------------------------------------------------------------------
 
-static CGFloat ScreenMax(NSWindow* win)
+static CGFloat ScreenMax()
 {
   return NSMaxY([[NSScreen mainScreen] frame]);
 }
@@ -1146,7 +1147,7 @@ PRectangle Window::GetPosition()
       win = reinterpret_cast<NSWindow*>(idWin);
       rect = [win frame];
     }
-    CGFloat screenHeight = ScreenMax(win);
+    CGFloat screenHeight = ScreenMax();
     // Invert screen positions to match Scintilla
     return PRectangle(
         static_cast<XYPOSITION>(NSMinX(rect)), static_cast<XYPOSITION>(screenHeight - NSMaxY(rect)),
@@ -1179,7 +1180,7 @@ void Window::SetPosition(PRectangle rc)
       // NSWindow
       PLATFORM_ASSERT([idWin isKindOfClass: [NSWindow class]]);
       NSWindow* win = reinterpret_cast<NSWindow*>(idWin);
-      CGFloat screenHeight = ScreenMax(win);
+      CGFloat screenHeight = ScreenMax();
       NSRect nsrc = NSMakeRect(rc.left, screenHeight - rc.bottom,
           rc.Width(), rc.Height());
       [win setFrame: nsrc display:YES];
@@ -1298,7 +1299,7 @@ void Window::SetCursor(Cursor curs)
   if (wid)
   {
     id idWin = reinterpret_cast<id>(wid);
-    if ([idWin isMemberOfClass: [SCIContentView class]])
+    if ([idWin isKindOfClass: [SCIContentView class]])
     {
       SCIContentView* container = reinterpret_cast<SCIContentView*>(idWin);
       [container setCursor: curs];
@@ -1719,6 +1720,7 @@ int ListBoxImpl::CaretFromEdge()
 
 void ListBoxImpl::ReleaseViews()
 {
+  [table setDataSource:nil];
   [table release];
   table = nil;
   [scroller release];
