@@ -198,6 +198,12 @@ static const KeyToCommand macMapDefault[] =
 
 }
 
+- (void) dealloc {
+	[sFind release];
+	[sFont release];
+	[super dealloc];
+}
+
 const CGFloat paddingHighlightX = 4;
 const CGFloat paddingHighlightY = 2;
 
@@ -333,6 +339,18 @@ const CGFloat paddingHighlightY = 2;
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Method called by owning ScintillaCocoa object when it is destroyed.
+ */
+- (void) ownerDestroyed
+{
+  mTarget = NULL;
+  [notificationQueue release];
+  notificationQueue = nil;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Method called by a timer installed by ScintillaCocoa. This two step approach is needed because
  * a native Obj-C class is required as target for the timer.
@@ -358,7 +376,7 @@ const CGFloat paddingHighlightY = 2;
   [notificationQueue enqueueNotification: notification
                             postingStyle: NSPostWhenIdle
                             coalesceMask: (NSNotificationCoalescingOnName | NSNotificationCoalescingOnSender)
-                                forModes: nil];
+                                forModes: @[NSDefaultRunLoopMode, NSModalPanelRunLoopMode]];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -377,11 +395,15 @@ const CGFloat paddingHighlightY = 2;
 
 //----------------- ScintillaCocoa -----------------------------------------------------------------
 
-ScintillaCocoa::ScintillaCocoa(SCIContentView* view, SCIMarginView* viewMargin)
+ScintillaCocoa::ScintillaCocoa(ScintillaView* sciView_, SCIContentView* viewContent, SCIMarginView* viewMargin)
 {
   vs.marginInside = false;
-  wMain = view; // Don't retain since we're owned by view, which would cause a cycle
+
+  // Don't retain since we're owned by view, which would cause a cycle
+  sciView = sciView_;
+  wMain = viewContent;
   wMargin = viewMargin;
+
   timerTarget = [[TimerTarget alloc] init: this];
   lastMouseEvent = NULL;
   delegate = NULL;
@@ -391,8 +413,6 @@ ScintillaCocoa::ScintillaCocoa(SCIContentView* view, SCIMarginView* viewMargin)
   enteredSetScrollingSize = false;
   scrollSpeed = 1;
   scrollTicks = 2000;
-  tickTimer = NULL;
-  idleTimer = NULL;
   observer = NULL;
   layerFindIndicator = NULL;
   imeInteraction = imeInline;
@@ -408,6 +428,7 @@ ScintillaCocoa::ScintillaCocoa(SCIContentView* view, SCIMarginView* viewMargin)
 ScintillaCocoa::~ScintillaCocoa()
 {
   Finalise();
+  [timerTarget ownerDestroyed];
   [timerTarget release];
 }
 
@@ -534,7 +555,7 @@ static std::string EncodedBytesString(CFStringRef cfsRef, CFStringEncoding encod
 	CFIndex usedLen = 0;
 	CFStringGetBytes(cfsRef, rangeAll, encoding, '?', false,
                          NULL, 0, &usedLen);
-	
+
 	std::string buffer(usedLen, '\0');
 	if (usedLen > 0) {
 		CFStringGetBytes(cfsRef, rangeAll, encoding, '?', false,
@@ -560,24 +581,24 @@ public:
 			folded[0] = mapping[static_cast<unsigned char>(mixed[0])];
 			return 1;
 		} else {
-            CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
-                                                         reinterpret_cast<const UInt8 *>(mixed),
-                                                         lenMixed, encoding, false);
+			CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
+								     reinterpret_cast<const UInt8 *>(mixed),
+								     lenMixed, encoding, false);
 
-            NSString *sMapped = [(NSString *)cfsVal stringByFoldingWithOptions:NSCaseInsensitiveSearch
-                                                                        locale:[NSLocale currentLocale]];
+			NSString *sMapped = [(NSString *)cfsVal stringByFoldingWithOptions:NSCaseInsensitiveSearch
+										    locale:[NSLocale currentLocale]];
 
-            char *encoded = EncodedBytes((CFStringRef)sMapped, encoding);
+			char *encoded = EncodedBytes((CFStringRef)sMapped, encoding);
 
 			size_t lenMapped = strlen(encoded);
-            if (lenMapped < sizeFolded) {
-                memcpy(folded, encoded,  lenMapped);
-            } else {
-                folded[0] = '\0';
-                lenMapped = 1;
-            }
-            delete []encoded;
-            CFRelease(cfsVal);
+			if (lenMapped < sizeFolded) {
+				memcpy(folded, encoded,  lenMapped);
+			} else {
+				folded[0] = '\0';
+				lenMapped = 1;
+			}
+			delete []encoded;
+			CFRelease(cfsVal);
 			return lenMapped;
 		}
 	}
@@ -587,38 +608,37 @@ CaseFolder *ScintillaCocoa::CaseFolderForEncoding() {
 	if (pdoc->dbcsCodePage == SC_CP_UTF8) {
 		return new CaseFolderUnicode();
 	} else {
-        CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
-                                                             vs.styles[STYLE_DEFAULT].characterSet);
-        if (pdoc->dbcsCodePage == 0) {
-            CaseFolderTable *pcf = new CaseFolderTable();
-            pcf->StandardASCII();
-            // Only for single byte encodings
-            for (int i=0x80; i<0x100; i++) {
-                char sCharacter[2] = "A";
-                sCharacter[0] = static_cast<char>(i);
-                CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
-                                                             reinterpret_cast<const UInt8 *>(sCharacter),
-                                                             1, encoding, false);
-                if (!cfsVal)
-                        continue;
+		CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
+								     vs.styles[STYLE_DEFAULT].characterSet);
+		if (pdoc->dbcsCodePage == 0) {
+			CaseFolderTable *pcf = new CaseFolderTable();
+			pcf->StandardASCII();
+			// Only for single byte encodings
+			for (int i=0x80; i<0x100; i++) {
+				char sCharacter[2] = "A";
+				sCharacter[0] = static_cast<char>(i);
+				CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
+									     reinterpret_cast<const UInt8 *>(sCharacter),
+									     1, encoding, false);
+				if (!cfsVal)
+					continue;
 
-                NSString *sMapped = [(NSString *)cfsVal stringByFoldingWithOptions:NSCaseInsensitiveSearch
-                                                                            locale:[NSLocale currentLocale]];
+				NSString *sMapped = [(NSString *)cfsVal stringByFoldingWithOptions:NSCaseInsensitiveSearch
+											    locale:[NSLocale currentLocale]];
 
-                char *encoded = EncodedBytes((CFStringRef)sMapped, encoding);
+				char *encoded = EncodedBytes((CFStringRef)sMapped, encoding);
 
-                if (strlen(encoded) == 1) {
-                    pcf->SetTranslation(sCharacter[0], encoded[0]);
-                }
+				if (strlen(encoded) == 1) {
+					pcf->SetTranslation(sCharacter[0], encoded[0]);
+				}
 
-                delete []encoded;
-                CFRelease(cfsVal);
-            }
-            return pcf;
-        } else {
-            return new CaseFolderDBCS(encoding);
-        }
-		return 0;
+				delete []encoded;
+				CFRelease(cfsVal);
+			}
+			return pcf;
+		} else {
+			return new CaseFolderDBCS(encoding);
+		}
 	}
 }
 
@@ -842,45 +862,51 @@ sptr_t ScintillaCocoa::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lPar
     {
       case SCI_GETDIRECTFUNCTION:
         return reinterpret_cast<sptr_t>(DirectFunction);
-        
+
       case SCI_GETDIRECTPOINTER:
         return reinterpret_cast<sptr_t>(this);
-        
+
       case SCI_TARGETASUTF8:
         return TargetAsUTF8(reinterpret_cast<char*>(lParam));
-        
+
       case SCI_ENCODEDFROMUTF8:
         return EncodedFromUTF8(reinterpret_cast<char*>(wParam),
                                reinterpret_cast<char*>(lParam));
-        
+
       case SCI_SETIMEINTERACTION:
         // Only inline IME supported on Cocoa
         break;
-        
+
       case SCI_GRABFOCUS:
         [[ContentView() window] makeFirstResponder:ContentView()];
         break;
-        
+
       case SCI_SETBUFFEREDDRAW:
         // Buffered drawing not supported on Cocoa
         view.bufferedDraw = false;
         break;
-        
+
       case SCI_FINDINDICATORSHOW:
         ShowFindIndicatorForRange(NSMakeRange(wParam, lParam-wParam), YES);
         return 0;
-        
+
       case SCI_FINDINDICATORFLASH:
         ShowFindIndicatorForRange(NSMakeRange(wParam, lParam-wParam), NO);
         return 0;
-        
+
       case SCI_FINDINDICATORHIDE:
         HideFindIndicator();
         return 0;
-        
+
+      case SCI_SETPHASESDRAW: {
+          sptr_t r = ScintillaBase::WndProc(iMessage, wParam, lParam);
+          [sciView updateIndicatorIME];
+          return r;
+        }
+
       default:
         sptr_t r = ScintillaBase::WndProc(iMessage, wParam, lParam);
-        
+
         return r;
     }
   } catch (std::bad_alloc &) {
@@ -947,16 +973,18 @@ bool ScintillaCocoa::FineTickerRunning(TickReason reason)
 void ScintillaCocoa::FineTickerStart(TickReason reason, int millis, int tolerance)
 {
   FineTickerCancel(reason);
-  NSTimer *fineTimer = [NSTimer scheduledTimerWithTimeInterval: millis / 1000.0
-                                                        target: timerTarget
-                                                      selector: @selector(timerFired:)
-                                                      userInfo: nil
-                                                       repeats: YES];
+  NSTimer *fineTimer = [NSTimer timerWithTimeInterval: millis / 1000.0
+                                               target: timerTarget
+                                             selector: @selector(timerFired:)
+                                             userInfo: nil
+                                              repeats: YES];
   if (tolerance && [fineTimer respondsToSelector: @selector(setTolerance:)])
   {
     [fineTimer setTolerance: tolerance / 1000.0];
   }
   timers[reason] = fineTimer;
+  [NSRunLoop.currentRunLoop addTimer: fineTimer forMode: NSDefaultRunLoopMode];
+  [NSRunLoop.currentRunLoop addTimer: fineTimer forMode: NSModalPanelRunLoopMode];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -983,11 +1011,12 @@ bool ScintillaCocoa::SetIdle(bool on)
     if (idler.state)
     {
       // Scintilla ticks = milliseconds
-      idleTimer = [NSTimer scheduledTimerWithTimeInterval: timer.tickSize / 1000.0
-						   target: timerTarget
-						 selector: @selector(idleTimerFired:)
-						 userInfo: nil
-						  repeats: YES];
+      NSTimer *idleTimer = [NSTimer scheduledTimerWithTimeInterval: timer.tickSize / 1000.0
+                                                            target: timerTarget
+                                                          selector: @selector(idleTimerFired:)
+                                                          userInfo: nil
+                                                           repeats: YES];
+      [NSRunLoop.currentRunLoop addTimer: idleTimer forMode: NSModalPanelRunLoopMode];
       idler.idlerID = reinterpret_cast<IdlerID>(idleTimer);
     }
     else
@@ -1271,16 +1300,17 @@ void ScintillaCocoa::DragScroll()
 - (id)initWithSelectedText:(const SelectionText *)other
 {
   self = [super init];
-  
+
   if (self) {
     selectedText.Copy(*other);
   }
-  
+
   return self;
 }
 
 - (void)pasteboard:(NSPasteboard *)pasteboard item:(NSPasteboardItem *)item provideDataForType:(NSString *)type
 {
+#pragma unused(item)
   if (selectedText.Length() == 0)
     return;
 
@@ -1293,7 +1323,7 @@ void ScintillaCocoa::DragScroll()
   CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
                                                reinterpret_cast<const UInt8 *>(selectedText.Data()),
                                                selectedText.Length(), encoding, false);
-  
+
   if ([type compare: NSPasteboardTypeString] == NSOrderedSame)
   {
     [pasteboard setString:(NSString *)cfsVal forType: NSStringPboardType];
@@ -1306,7 +1336,7 @@ void ScintillaCocoa::DragScroll()
   }
 
   if (cfsVal)
-    CFRelease(cfsVal);  
+    CFRelease(cfsVal);
 }
 
 @end
@@ -1407,59 +1437,40 @@ void ScintillaCocoa::StartDrag()
   SCIContentView* content = ContentView();
 
   // To get a bitmap of the text we're dragging, we just use Paint on a pixmap surface.
-  SurfaceImpl *sw = new SurfaceImpl();
-  SurfaceImpl *pixmap = NULL;
+  SurfaceImpl sw;
+  sw.InitPixMap(static_cast<int>(client.Width()), static_cast<int>(client.Height()), NULL, NULL);
 
-  bool lastHideSelection = view.hideSelection;
+  const bool lastHideSelection = view.hideSelection;
   view.hideSelection = true;
-  if (sw)
-  {
-    pixmap = new SurfaceImpl();
-    if (pixmap)
-    {
-      PRectangle imageRect = rcSel;
-      paintState = painting;
-      sw->InitPixMap(static_cast<int>(client.Width()), static_cast<int>(client.Height()), NULL, NULL);
-      paintingAllText = true;
-      // Have to create a new context and make current as text drawing goes
-      // to the current context, not a passed context.
-      CGContextRef gcsw = sw->GetContext();
-      NSGraphicsContext *nsgc = [NSGraphicsContext graphicsContextWithGraphicsPort: gcsw
-                                                                           flipped: YES];
-      [NSGraphicsContext setCurrentContext:nsgc];
-      CGContextTranslateCTM(gcsw, -client.left, -client.top);
-      Paint(sw, client);
-      paintState = notPainting;
-
-      pixmap->InitPixMap(static_cast<int>(imageRect.Width()), static_cast<int>(imageRect.Height()), NULL, NULL);
-      pixmap->SetUnicodeMode(IsUnicodeMode());
-      pixmap->SetDBCSMode(CodePage());
-
-      CGContextRef gc = pixmap->GetContext();
-      // To make Paint() work on a bitmap, we have to flip our coordinates and translate the origin
-      CGContextTranslateCTM(gc, 0, imageRect.Height());
-      CGContextScaleCTM(gc, 1.0, -1.0);
-
-      pixmap->CopyImageRectangle(*sw, imageRect, PRectangle(0.0f, 0.0f, imageRect.Width(), imageRect.Height()));
-      // XXX TODO: overwrite any part of the image that is not part of the
-      //           selection to make it transparent.  right now we just use
-      //           the full rectangle which may include non-selected text.
-    }
-    sw->Release();
-    delete sw;
-  }
+  PRectangle imageRect = rcSel;
+  paintState = painting;
+  paintingAllText = true;
+  CGContextRef gcsw = sw.GetContext();
+  CGContextTranslateCTM(gcsw, -client.left, -client.top);
+  Paint(&sw, client);
+  paintState = notPainting;
   view.hideSelection = lastHideSelection;
 
+  SurfaceImpl pixmap;
+  pixmap.InitPixMap(static_cast<int>(imageRect.Width()), static_cast<int>(imageRect.Height()), NULL, NULL);
+  pixmap.SetUnicodeMode(IsUnicodeMode());
+  pixmap.SetDBCSMode(CodePage());
+
+  CGContextRef gc = pixmap.GetContext();
+  // To make Paint() work on a bitmap, we have to flip our coordinates and translate the origin
+  CGContextTranslateCTM(gc, 0, imageRect.Height());
+  CGContextScaleCTM(gc, 1.0, -1.0);
+
+  pixmap.CopyImageRectangle(sw, imageRect, PRectangle(0.0f, 0.0f, imageRect.Width(), imageRect.Height()));
+  // XXX TODO: overwrite any part of the image that is not part of the
+  //           selection to make it transparent.  right now we just use
+  //           the full rectangle which may include non-selected text.
+
   NSBitmapImageRep* bitmap = NULL;
-  if (pixmap)
-  {
-    CGImageRef imagePixmap = pixmap->GetImage();
-    if (imagePixmap)
-      bitmap = [[[NSBitmapImageRep alloc] initWithCGImage: imagePixmap] autorelease];
-    CGImageRelease(imagePixmap);
-    pixmap->Release();
-    delete pixmap;
-  }
+  CGImageRef imagePixmap = pixmap.GetImage();
+  if (imagePixmap)
+    bitmap = [[[NSBitmapImageRep alloc] initWithCGImage: imagePixmap] autorelease];
+  CGImageRelease(imagePixmap);
 
   NSImage* image = [[[NSImage alloc] initWithSize: selectionRectangle.size] autorelease];
   [image addRepresentation: bitmap];
@@ -1473,17 +1484,17 @@ void ScintillaCocoa::StartDrag()
   NSPoint startPoint;
   startPoint.x = selectionRectangle.origin.x + client.left;
   startPoint.y = selectionRectangle.origin.y + selectionRectangle.size.height + client.top;
-  
+
   NSPasteboardItem *pbItem = [NSPasteboardItem new];
   DragProviderSource *dps = [[[DragProviderSource alloc] initWithSelectedText:&selectedText] autorelease];
-  
+
   NSArray *pbTypes = selectedText.rectangular ?
   @[NSPasteboardTypeString, ScintillaRecPboardType] :
   @[NSPasteboardTypeString];
   [pbItem setDataProvider:dps forTypes:pbTypes];
   NSDraggingItem *dragItem = [[NSDraggingItem alloc ]initWithPasteboardWriter:pbItem];
   [pbItem release];
-  
+
   NSScrollView *scrollContainer = ScrollContainer();
   NSRect contentRect = [[scrollContainer contentView] bounds];
   NSRect draggingRect = NSOffsetRect(selectionRectangle, contentRect.origin.x, contentRect.origin.y);
@@ -1675,24 +1686,97 @@ int ScintillaCocoa::TargetAsUTF8(char *text)
     CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
                                                  reinterpret_cast<const UInt8 *>(s.c_str()),
                                                  s.length(), encoding, false);
-	  
+
     const std::string tmputf = EncodedBytesString(cfsVal, kCFStringEncodingUTF8);
-    
+
     if (text)
       memcpy(text, tmputf.c_str(), tmputf.length());
     CFRelease(cfsVal);
-    return tmputf.length();
+    return static_cast<int>(tmputf.length());
   }
   return targetLength;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+// Returns the text in the range converted to an NSString.
+NSString *ScintillaCocoa::RangeTextAsString(NSRange rangePositions) const {
+  const std::string text = RangeText(static_cast<int>(rangePositions.location),
+				     static_cast<int>(NSMaxRange(rangePositions)));
+  if (IsUnicodeMode())
+  {
+    return [NSString stringWithUTF8String: text.c_str()];
+  }
+  else
+  {
+    // Need to convert
+    const CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
+							       vs.styles[STYLE_DEFAULT].characterSet);
+    CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
+						 reinterpret_cast<const UInt8 *>(text.c_str()),
+						 text.length(), encoding, false);
+
+    return (NSString *)cfsVal;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+// Return character range of a line.
+NSRange ScintillaCocoa::RangeForVisibleLine(NSInteger lineVisible) {
+  const Range posRangeLine = RangeDisplayLine(static_cast<int>(lineVisible));
+  return CharactersFromPositions(NSMakeRange(posRangeLine.First(),
+					     posRangeLine.Last() - posRangeLine.First()));
+}
+
+//--------------------------------------------------------------------------------------------------
+
+// Returns visible line number of a text position in characters.
+NSInteger ScintillaCocoa::VisibleLineForIndex(NSInteger index) {
+  const NSRange rangePosition = PositionsFromCharacters(NSMakeRange(index, 0));
+  const int lineVisible = DisplayFromPosition(static_cast<int>(rangePosition.location));
+  return lineVisible;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+// Returns a rectangle that frames the range for use by the VoiceOver cursor.
+NSRect ScintillaCocoa::FrameForRange(NSRange rangeCharacters) {
+  const NSRange posRange = PositionsFromCharacters(rangeCharacters);
+
+  NSUInteger rangeEnd = NSMaxRange(posRange);
+  const bool endsWithLineEnd = rangeCharacters.length &&
+    (pdoc->GetColumn(static_cast<int>(rangeEnd)) == 0);
+
+  Point ptStart = LocationFromPosition(static_cast<int>(posRange.location));
+  const PointEnd peEndRange = static_cast<PointEnd>(peSubLineEnd|peLineEnd);
+  Point ptEnd = LocationFromPosition(static_cast<int>(rangeEnd), peEndRange);
+
+  NSRect rect = NSMakeRect(ptStart.x, ptStart.y,
+			   ptEnd.x - ptStart.x,
+			   ptEnd.y - ptStart.y);
+
+  rect.size.width += 2;	// Shows the last character better
+  if (endsWithLineEnd) {
+    // Add a block to the right to indicate a line end is selected
+    rect.size.width += 20;
+  }
+
+  rect.size.height += vs.lineHeight;
+
+  // Adjust for margin and scroll
+  rect.origin.x = rect.origin.x - vs.textStart + vs.fixedColumnWidth;
+
+  return rect;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 // Translates a UTF8 string into the document encoding.
 // Return the length of the result in bytes.
 int ScintillaCocoa::EncodedFromUTF8(char *utf8, char *encoded) const
 {
-  const int inputLength = (lengthForEncode >= 0) ? lengthForEncode : strlen(utf8);
+  const int inputLength = (lengthForEncode >= 0) ? lengthForEncode : static_cast<int>(strlen(utf8));
   if (IsUnicodeMode())
   {
     if (encoded)
@@ -1704,7 +1788,7 @@ int ScintillaCocoa::EncodedFromUTF8(char *utf8, char *encoded) const
     // Need to convert
     const CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
                                                          vs.styles[STYLE_DEFAULT].characterSet);
-    
+
     CFStringRef cfsVal = CFStringCreateWithBytes(kCFAllocatorDefault,
                                                  reinterpret_cast<const UInt8 *>(utf8),
                                                  inputLength, kCFStringEncodingUTF8, false);
@@ -1712,7 +1796,7 @@ int ScintillaCocoa::EncodedFromUTF8(char *utf8, char *encoded) const
     if (encoded)
       memcpy(encoded, sEncoded.c_str(), sEncoded.length());
     CFRelease(cfsVal);
-    return sEncoded.length();
+    return static_cast<int>(sEncoded.length());
   }
 }
 
@@ -1806,13 +1890,15 @@ void ScintillaCocoa::WillDraw(NSRect rect)
 {
   RefreshStyleData();
   PRectangle rcWillDraw = NSRectToPRectangle(rect);
-  int positionAfterRect = PositionAfterArea(rcWillDraw);
-  pdoc->EnsureStyledTo(positionAfterRect);
+  const int posAfterArea = PositionAfterArea(rcWillDraw);
+  const int posAfterMax = PositionAfterMaxStyling(posAfterArea, true);
+  pdoc->StyleToAdjustingLineDuration(posAfterMax);
+  StartIdleStyling(posAfterMax < posAfterArea);
   NotifyUpdateUI();
   if (WrapLines(wsVisible)) {
     // Wrap may have reduced number of lines so more lines may need to be styled
-    positionAfterRect = PositionAfterArea(rcWillDraw);
-    pdoc->EnsureStyledTo(positionAfterRect);
+    const int posAfterAreaWrapped = PositionAfterArea(rcWillDraw);
+    pdoc->EnsureStyledTo(posAfterAreaWrapped);
     // The wrapping process has changed the height of some lines so redraw all.
     Redraw();
   }
@@ -1917,7 +2003,7 @@ bool ScintillaCocoa::SetScrollingSize(void) {
 		SetVerticalScrollPos();
 		enteredSetScrollingSize = false;
 	}
-	[inner.owner setMarginWidth: vs.fixedColumnWidth];
+	[sciView setMarginWidth: vs.fixedColumnWidth];
 	return changes;
 }
 
@@ -2010,6 +2096,15 @@ void ScintillaCocoa::NotifyParent(SCNotification scn)
     notifyProc(notifyObj, WM_NOTIFY, GetCtrlID(), (uintptr_t) &scn);
   if (delegate)
     [delegate notification:&scn];
+  if (scn.nmhdr.code == SCN_UPDATEUI) {
+    NSView *content = ContentView();
+    if (scn.updated & SC_UPDATE_CONTENT) {
+      NSAccessibilityPostNotification(content, NSAccessibilityValueChangedNotification);
+    }
+    if (scn.updated & SC_UPDATE_SELECTION) {
+      NSAccessibilityPostNotification(content, NSAccessibilitySelectedTextChangedNotification);
+    }
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -2084,7 +2179,7 @@ bool ScintillaCocoa::Draw(NSRect rect, CGContextRef gc)
 /**
  * Helper function to translate OS X key codes to Scintilla key codes.
  */
-static inline UniChar KeyTranslate(UniChar unicodeChar)
+static inline UniChar KeyTranslate(UniChar unicodeChar, NSEventModifierFlags modifierFlags)
 {
   switch (unicodeChar)
   {
@@ -2113,6 +2208,21 @@ static inline UniChar KeyTranslate(UniChar unicodeChar)
       return SCK_RETURN;
     case 27:
       return SCK_ESCAPE;
+    case '+':
+      if (modifierFlags & NSNumericPadKeyMask)
+        return SCK_ADD;
+      else
+        return unicodeChar;
+    case '-':
+      if (modifierFlags & NSNumericPadKeyMask)
+        return SCK_SUBTRACT;
+      else
+        return unicodeChar;
+    case '/':
+      if (modifierFlags & NSNumericPadKeyMask)
+        return SCK_DIVIDE;
+      else
+        return unicodeChar;
     case 127:
       return SCK_BACK;
     case '\t':
@@ -2153,7 +2263,7 @@ static int TranslateModifierFlags(NSUInteger modifiers)
 bool ScintillaCocoa::KeyboardInput(NSEvent* event)
 {
   // For now filter out function keys.
-  NSString* input = [event characters];
+  NSString* input = [event charactersIgnoringModifiers];
 
   bool handled = false;
 
@@ -2161,11 +2271,13 @@ bool ScintillaCocoa::KeyboardInput(NSEvent* event)
   for (size_t i = 0; i < input.length; i++)
   {
     const UniChar originalKey = [input characterAtIndex: i];
-    UniChar key = KeyTranslate(originalKey);
+    NSEventModifierFlags modifierFlags = [event modifierFlags];
+      
+    UniChar key = KeyTranslate(originalKey, modifierFlags);
 
     bool consumed = false; // Consumed as command?
 
-    if (KeyDownWithModifiers(key, TranslateModifierFlags([event modifierFlags]), &consumed))
+    if (KeyDownWithModifiers(key, TranslateModifierFlags(modifierFlags), &consumed))
       handled = true;
     if (consumed)
       handled = true;
@@ -2184,7 +2296,7 @@ int ScintillaCocoa::InsertText(NSString* input)
   CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
                                                        vs.styles[STYLE_DEFAULT].characterSet);
   std::string encoded = EncodedBytesString((CFStringRef)input, encoding);
-  
+
   if (encoded.length() > 0)
   {
     AddCharUTF((char*) encoded.c_str(), static_cast<unsigned int>(encoded.length()), false);
@@ -2197,12 +2309,12 @@ int ScintillaCocoa::InsertText(NSString* input)
 /**
  * Convert from a range of characters to a range of bytes.
  */
-NSRange ScintillaCocoa::PositionsFromCharacters(NSRange range) const
+NSRange ScintillaCocoa::PositionsFromCharacters(NSRange rangeCharacters) const
 {
-  long start = pdoc->GetRelativePositionUTF16(0, range.location);
+  long start = pdoc->GetRelativePositionUTF16(0, static_cast<int>(rangeCharacters.location));
   if (start == INVALID_POSITION)
     start = pdoc->Length();
-  long end = pdoc->GetRelativePositionUTF16(start, range.length);
+  long end = pdoc->GetRelativePositionUTF16(static_cast<int>(start), static_cast<int>(rangeCharacters.length));
   if (end == INVALID_POSITION)
     end = pdoc->Length();
   return NSMakeRange(start, end - start);
@@ -2213,10 +2325,11 @@ NSRange ScintillaCocoa::PositionsFromCharacters(NSRange range) const
 /**
  * Convert from a range of characters from a range of bytes.
  */
-NSRange ScintillaCocoa::CharactersFromPositions(NSRange range) const
+NSRange ScintillaCocoa::CharactersFromPositions(NSRange rangePositions) const
 {
-  const long start = pdoc->CountUTF16(0, range.location);
-  const long len = pdoc->CountUTF16(range.location, NSMaxRange(range));
+  const long start = pdoc->CountUTF16(0, static_cast<int>(rangePositions.location));
+  const long len = pdoc->CountUTF16(static_cast<int>(rangePositions.location),
+				    static_cast<int>(NSMaxRange(rangePositions)));
   return NSMakeRange(start, len);
 }
 
@@ -2239,7 +2352,7 @@ void ScintillaCocoa::SelectOnlyMainSelection()
  */
 void ScintillaCocoa::ConvertSelectionVirtualSpace()
 {
-  FillVirtualSpace();
+  ClearBeforeTentativeStart();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -2342,6 +2455,17 @@ void ScintillaCocoa::MouseDown(NSEvent* event)
   ButtonDown(Point(location.x, location.y), (int) (time * 1000), shift, command, alt);
 }
 
+void ScintillaCocoa::RightMouseDown(NSEvent *event)
+{
+  Point location = ConvertPoint([event locationInWindow]);
+  NSTimeInterval time = [event timestamp];
+  bool command = ([event modifierFlags] & NSCommandKeyMask) != 0;
+  bool shift = ([event modifierFlags] & NSShiftKeyMask) != 0;
+  bool alt = ([event modifierFlags] & NSAlternateKeyMask) != 0;
+
+  RightButtonDownWithModifiers(Point(location.x, location.y), (int) (time * 1000), ModifierFlags(shift, command, alt));
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void ScintillaCocoa::MouseMove(NSEvent* event)
@@ -2420,6 +2544,16 @@ void ScintillaCocoa::Redo()
 
 //--------------------------------------------------------------------------------------------------
 
+bool ScintillaCocoa::ShouldDisplayPopupOnMargin()
+{
+  return displayPopupMenu == SC_POPUP_ALL;
+}
+
+bool ScintillaCocoa::ShouldDisplayPopupOnText()
+{
+  return displayPopupMenu == SC_POPUP_ALL || displayPopupMenu == SC_POPUP_TEXT;
+}
+
 /**
  * Creates and returns a popup menu, which is then displayed by the Cocoa framework.
  */
