@@ -15,6 +15,12 @@
 
 using namespace Scintilla;
 
+// Add backend property to ScintillaView as a private category.
+// Specified here as backend accessed by SCIMarginView and SCIContentView.
+@interface ScintillaView ()
+@property (nonatomic, readonly) Scintilla::ScintillaCocoa* backend;
+@end
+
 // Two additional cursors we need, which aren't provided by Cocoa.
 static NSCursor* reverseArrowCursor;
 static NSCursor* waitCursor;
@@ -46,8 +52,17 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
   }
 }
 
+// Add marginWidth and owner properties as a private category.
+@interface SCIMarginView ()
+@property (assign) int marginWidth;
+@property (nonatomic, assign) ScintillaView* owner;
+@end
 
-@implementation SCIMarginView
+@implementation SCIMarginView {
+  int marginWidth;
+  ScintillaView *owner;
+  NSMutableArray *currentCursors;
+}
 
 @synthesize marginWidth, owner;
 
@@ -64,6 +79,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
       [currentCursors addObject: [reverseArrowCursor retain]];
     }
     [self setClientView:[aScrollView documentView]];
+    self.accessibilityLabel = @"Scintilla Margin";
   }
   return self;
 }
@@ -100,6 +116,21 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
   }
 }
 
+/**
+ * Called by the framework if it wants to show a context menu for the margin.
+ */
+- (NSMenu*) menuForEvent: (NSEvent*) theEvent
+{
+  NSMenu *menu = [owner menuForEvent: theEvent];
+  if (menu) {
+    return menu;
+  } else if (owner.backend->ShouldDisplayPopupOnMargin()) {
+    return owner.backend->CreateContextMenu(theEvent);
+  } else {
+    return nil;
+  }
+}
+
 - (void) mouseDown: (NSEvent *) theEvent
 {
   NSClipView *textView = [[self scrollView] contentView];
@@ -107,6 +138,13 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
   owner.backend->MouseDown(theEvent);
 }
 
+- (void) rightMouseDown: (NSEvent *) theEvent
+{
+  [NSMenu popUpContextMenu:[self menuForEvent: theEvent] withEvent:theEvent forView:self];
+
+  owner.backend->RightMouseDown(theEvent);
+}
+
 - (void) mouseDragged: (NSEvent *) theEvent
 {
   owner.backend->MouseMove(theEvent);
@@ -122,6 +160,12 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
   owner.backend->MouseUp(theEvent);
 }
 
+// Not a simple button so return failure
+- (BOOL)accessibilityPerformPress
+{
+  return NO;
+}
+
 /**
  * This method is called to give us the opportunity to define our mouse sensitive rectangle.
  */
@@ -148,7 +192,19 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 
 @end
 
-@implementation SCIContentView
+// Add owner property as a private category.
+@interface SCIContentView ()
+@property (nonatomic, assign) ScintillaView* owner;
+@end
+
+@implementation SCIContentView {
+  ScintillaView* mOwner;
+  NSCursor* mCurrentCursor;
+  NSTrackingArea *trackingArea;
+
+  // Set when we are in composition mode and partial input is displayed.
+  NSRange mMarkedTextRange;
+}
 
 @synthesize owner = mOwner;
 
@@ -167,6 +223,14 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 
     [self registerForDraggedTypes: [NSArray arrayWithObjects:
                                    NSStringPboardType, ScintillaRecPboardType, NSFilenamesPboardType, nil]];
+
+    // Set up accessibility in the text role
+    self.accessibilityElement = TRUE;
+    self.accessibilityEnabled = TRUE;
+    self.accessibilityLabel = NSLocalizedString(@"Scintilla", nil);	// No real localization
+    self.accessibilityRoleDescription = @"source code editor";
+    self.accessibilityRole = NSAccessibilityTextAreaRole;
+    self.accessibilityIdentifier = @"Scintilla";
   }
 
   return self;
@@ -181,7 +245,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 {
   if (trackingArea)
     [self removeTrackingArea:trackingArea];
-  
+
   int opts = (NSTrackingActiveAlways | NSTrackingInVisibleRect | NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved);
   trackingArea = [[NSTrackingArea alloc] initWithRect:[self bounds]
                                               options:opts
@@ -199,7 +263,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 - (void) setFrame: (NSRect) frame
 {
   [super setFrame: frame];
-	
+
   mOwner.backend->Resize();
 }
 
@@ -241,6 +305,11 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
  */
 - (void) viewWillDraw
 {
+  if (!mOwner) {
+    [super viewWillDraw];
+    return;
+  }
+
   const NSRect *rects;
   NSInteger nRects = 0;
   [self getRectsBeingDrawn:&rects count:&nRects];
@@ -261,7 +330,8 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
  */
 - (void) prepareContentInRect: (NSRect) rect
 {
-  mOwner.backend->WillDraw(rect);
+  if (mOwner)
+    mOwner.backend->WillDraw(rect);
 #if MAC_OS_X_VERSION_MAX_ALLOWED > 1080
   [super prepareContentInRect: rect];
 #endif
@@ -332,10 +402,14 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
  */
 - (NSMenu*) menuForEvent: (NSEvent*) theEvent
 {
-  if (![mOwner respondsToSelector: @selector(menuForEvent:)])
+  NSMenu *menu = [mOwner menuForEvent: theEvent];
+  if (menu) {
+    return menu;
+  } else if (mOwner.backend->ShouldDisplayPopupOnText()) {
     return mOwner.backend->CreateContextMenu(theEvent);
-  else
-    return [mOwner menuForEvent: theEvent];
+  } else {
+    return nil;
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -344,6 +418,10 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 
 - (NSAttributedString *)attributedSubstringForProposedRange:(NSRange)aRange actualRange:(NSRangePointer)actualRange
 {
+  const NSInteger lengthCharacters = self.accessibilityNumberOfCharacters;
+  if (aRange.location > lengthCharacters) {
+    return nil;
+  }
   const NSRange posRange = mOwner.backend->PositionsFromCharacters(aRange);
   // The backend validated aRange and may have removed characters beyond the end of the document.
   const NSRange charRange = mOwner.backend->CharactersFromPositions(posRange);
@@ -404,6 +482,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 
 - (NSRect) firstRectForCharacterRange: (NSRange) aRange actualRange: (NSRangePointer) actualRange
 {
+#pragma unused(actualRange)
   const NSRange posRange = mOwner.backend->PositionsFromCharacters(aRange);
 
   NSRect rect;
@@ -483,9 +562,11 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 
 - (NSRange) selectedRange
 {
-  const long positionBegin = [mOwner message: SCI_GETSELECTIONSTART];
-  const long positionEnd = [mOwner message: SCI_GETSELECTIONEND];
-  NSRange posRangeSel = NSMakeRange(positionBegin, positionEnd-positionBegin);
+  const NSRange posRangeSel = [mOwner selectedRangePositions];
+  if (posRangeSel.length == 0)
+  {
+    return NSMakeRange(NSNotFound, 0);
+  }
   return mOwner.backend->CharactersFromPositions(posRangeSel);
 }
 
@@ -530,7 +611,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
     // Must perform deletion before entering composition mode or else
     // both document and undo history will not contain the deleted text
     // leading to an inaccurate and unusable undo history.
-    
+
     // Convert selection virtual space into real space
     mOwner.backend->ConvertSelectionVirtualSpace();
 
@@ -549,7 +630,8 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
       }
       // Ensure only a single selection.
       mOwner.backend->SelectOnlyMainSelection();
-      replacementRange = [self selectedRange];
+      const NSRange posRangeSel = [mOwner selectedRangePositions];
+      replacementRange = mOwner.backend->CharactersFromPositions(posRangeSel);
     }
   }
 
@@ -561,7 +643,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
   {
     // Switching into composition.
     mOwner.backend->CompositionStart();
-    
+
     NSRange posRangeCurrent = mOwner.backend->PositionsFromCharacters(NSMakeRange(replacementRange.location, 0));
     // Note: Scintilla internally works almost always with bytes instead chars, so we need to take
     //       this into account when determining selection ranges and such.
@@ -606,7 +688,7 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 
 - (NSArray*) validAttributesForMarkedText
 {
-  return nil;
+  return @[];
 }
 
 // End of the NSTextInputClient protocol adoption.
@@ -694,6 +776,8 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
  */
 - (NSRect)adjustScroll:(NSRect)proposedVisibleRect
 {
+  if (!mOwner)
+    return proposedVisibleRect;
   NSRect rc = proposedVisibleRect;
   // Snap to lines
   NSRect contentRect = [self bounds];
@@ -737,11 +821,12 @@ static NSCursor *cursorFromEnum(Window::Cursor cursor)
 - (NSDragOperation)draggingSession: (NSDraggingSession *) session
 sourceOperationMaskForDraggingContext: (NSDraggingContext) context
 {
+#pragma unused(session)
   switch(context)
   {
     case NSDraggingContextOutsideApplication:
       return NSDragOperationCopy | NSDragOperationMove | NSDragOperationDelete;
-      
+
     case NSDraggingContextWithinApplication:
     default:
       return NSDragOperationCopy | NSDragOperationMove | NSDragOperationDelete;
@@ -751,12 +836,14 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
 - (void)draggingSession:(NSDraggingSession *)session
            movedToPoint:(NSPoint)screenPoint
 {
+#pragma unused(session, screenPoint)
 }
 
 - (void)draggingSession:(NSDraggingSession *)session
            endedAtPoint:(NSPoint)screenPoint
               operation:(NSDragOperation)operation
 {
+#pragma unused(session, screenPoint)
   if (operation == NSDragOperationDelete)
   {
     mOwner.backend->WndProc(SCI_CLEAR, 0, 0);
@@ -919,6 +1006,222 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
   return mOwner.backend->WndProc(SCI_GETREADONLY, 0, 0) == 0;
 }
 
+#pragma mark - NSAccessibility
+
+//--------------------------------------------------------------------------------------------------
+
+// Adoption of NSAccessibility protocol.
+// NSAccessibility wants to pass ranges in UTF-16 code units, not bytes (like Scintilla)
+// or characters.
+// Needs more testing with non-ASCII and non-BMP text.
+// Needs to take account of folding and wraping.
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Text of the whole document as a string.
+ */
+- (id) accessibilityValue {
+  const sptr_t length = [mOwner message: SCI_GETLENGTH];
+  return mOwner.backend->RangeTextAsString(NSMakeRange(0,static_cast<int>(length)));
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Line of the caret.
+ */
+- (NSInteger) accessibilityInsertionPointLineNumber {
+  const int caret = static_cast<int>([mOwner message: SCI_GETCURRENTPOS]);
+  const NSRange rangeCharactersCaret = mOwner.backend->CharactersFromPositions(NSMakeRange(caret, 0));
+  return mOwner.backend->VisibleLineForIndex(rangeCharactersCaret.location);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Not implemented and not called by VoiceOver.
+ */
+- (NSRange)accessibilityRangeForPosition:(NSPoint)point {
+  return NSMakeRange(0,0);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Number of characters in the whole document.
+ */
+- (NSInteger) accessibilityNumberOfCharacters {
+  sptr_t length = [mOwner message: SCI_GETLENGTH];
+  const NSRange posRange = mOwner.backend->CharactersFromPositions(NSMakeRange(length, 0));
+  return posRange.location;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : The selection text as a string.
+ */
+- (NSString *) accessibilitySelectedText {
+  const sptr_t positionBegin = [mOwner message: SCI_GETSELECTIONSTART];
+  const sptr_t positionEnd = [mOwner message: SCI_GETSELECTIONEND];
+  const NSRange posRangeSel = NSMakeRange(positionBegin, positionEnd-positionBegin);
+  return mOwner.backend->RangeTextAsString(posRangeSel);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : The character range of the main selection.
+ */
+- (NSRange) accessibilitySelectedTextRange {
+  const sptr_t positionBegin = [mOwner message: SCI_GETSELECTIONSTART];
+  const sptr_t positionEnd = [mOwner message: SCI_GETSELECTIONEND];
+  const NSRange posRangeSel = NSMakeRange(positionBegin, positionEnd-positionBegin);
+  return mOwner.backend->CharactersFromPositions(posRangeSel);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : The setter for accessibilitySelectedTextRange.
+ * This method is the only setter required for reasonable VoiceOver behaviour.
+ */
+- (void) setAccessibilitySelectedTextRange: (NSRange) range {
+  NSRange rangePositions = mOwner.backend->PositionsFromCharacters(range);
+  [mOwner message: SCI_SETSELECTION wParam: rangePositions.location lParam:NSMaxRange(rangePositions)];
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Range of the glyph at a character index.
+ * Currently doesn't try to handle composite characters.
+ */
+- (NSRange) accessibilityRangeForIndex: (NSInteger)index {
+  sptr_t length = [mOwner message: SCI_GETLENGTH];
+  const NSRange rangeLength = mOwner.backend->CharactersFromPositions(NSMakeRange(length, 0));
+  NSRange rangePositions = NSMakeRange(length, 0);
+  if (index < rangeLength.location) {
+    rangePositions = mOwner.backend->PositionsFromCharacters(NSMakeRange(index, 1));
+  }
+  return mOwner.backend->CharactersFromPositions(rangePositions);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : All the text ranges.
+ * Currently only returns the main selection.
+ */
+- (NSArray<NSValue *>*) accessibilitySelectedTextRanges {
+  const sptr_t positionBegin = [mOwner message: SCI_GETSELECTIONSTART];
+  const sptr_t positionEnd = [mOwner message: SCI_GETSELECTIONEND];
+  const NSRange posRangeSel = NSMakeRange(positionBegin, positionEnd-positionBegin);
+  NSRange rangeCharacters = mOwner.backend->CharactersFromPositions(posRangeSel);
+  NSValue *valueRange = [NSValue valueWithRange:(NSRange)rangeCharacters];
+  return @[valueRange];
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Character range currently visible.
+ */
+- (NSRange) accessibilityVisibleCharacterRange {
+  const sptr_t lineTopVisible = [mOwner message: SCI_GETFIRSTVISIBLELINE];
+  const sptr_t lineTop = [mOwner message:SCI_DOCLINEFROMVISIBLE wParam:lineTopVisible];
+  const sptr_t lineEndVisible = lineTopVisible + [mOwner message: SCI_LINESONSCREEN] - 1;
+  const sptr_t lineEnd = [mOwner message:SCI_DOCLINEFROMVISIBLE wParam:lineEndVisible];
+  const sptr_t posStartView = [mOwner message: SCI_POSITIONFROMLINE wParam: lineTop];
+  const sptr_t posEndView = [mOwner message: SCI_GETLINEENDPOSITION wParam: lineEnd];
+  const NSRange posRangeSel = NSMakeRange(posStartView, posEndView-posStartView);
+  return mOwner.backend->CharactersFromPositions(posRangeSel);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Character range of a line.
+ */
+- (NSRange)accessibilityRangeForLine:(NSInteger)line {
+  return mOwner.backend->RangeForVisibleLine(line);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Line number of a text position in characters.
+ */
+- (NSInteger)accessibilityLineForIndex:(NSInteger)index {
+  return mOwner.backend->VisibleLineForIndex(index);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : A rectangle that covers a range which will be shown as the
+ * VoiceOver cursor.
+ * Producing a nice rectangle is a little tricky particularly when including new
+ * lines. Needs to improve the case where parts of two lines are included.
+ */
+- (NSRect)accessibilityFrameForRange:(NSRange)range {
+  const NSRect rectInView = mOwner.backend->FrameForRange(range);
+  const NSRect rectInWindow = [[[self superview] superview] convertRect:rectInView toView:nil];
+  return [self.window convertRectToScreen:rectInWindow];
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : A range of text as a string.
+ */
+- (NSString *) accessibilityStringForRange:(NSRange)range {
+  const NSRange posRange = mOwner.backend->PositionsFromCharacters(range);
+  return mOwner.backend->RangeTextAsString(posRange);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : A range of text as an attributed string.
+ * Currently no attributes are set.
+ */
+- (NSAttributedString *) accessibilityAttributedStringForRange:(NSRange)range {
+  const NSRange posRange = mOwner.backend->PositionsFromCharacters(range);
+  NSString *result = mOwner.backend->RangeTextAsString(posRange);
+  return [[[NSMutableAttributedString alloc] initWithString:result] autorelease];
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * NSAccessibility : Show the context menu at the caret.
+ */
+- (BOOL)accessibilityPerformShowMenu {
+  const sptr_t caret = [mOwner message: SCI_GETCURRENTPOS];
+  NSRect rect;
+  rect.origin.x = [mOwner message: SCI_POINTXFROMPOSITION wParam: 0 lParam: caret];
+  rect.origin.y = [mOwner message: SCI_POINTYFROMPOSITION wParam: 0 lParam: caret];
+  rect.origin.y += [mOwner message: SCI_TEXTHEIGHT wParam: 0 lParam: 0];
+  rect.size.width = 1.0;
+  rect.size.height = 1.0;
+  NSRect rectInWindow = [[[self superview] superview] convertRect:rect toView:nil];
+  NSPoint pt = rectInWindow.origin;
+  NSEvent *event = [NSEvent mouseEventWithType: NSRightMouseDown
+				      location: pt
+				 modifierFlags: 0
+				     timestamp: 0
+				  windowNumber: [[self window] windowNumber]
+				       context: nil
+				   eventNumber: 0
+				    clickCount: 1
+				      pressure: 0.0];
+  NSMenu *menu = mOwner.backend->CreateContextMenu(event);
+  [NSMenu popUpContextMenu:menu withEvent:event forView:self];
+  return YES;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 - (void) dealloc
@@ -931,7 +1234,25 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
 
 //--------------------------------------------------------------------------------------------------
 
-@implementation ScintillaView
+@implementation ScintillaView {
+  // The back end is kind of a controller and model in one.
+  // It uses the content view for display.
+  Scintilla::ScintillaCocoa* mBackend;
+
+  // This is the actual content to which the backend renders itself.
+  SCIContentView* mContent;
+
+  NSScrollView *scrollView;
+  SCIMarginView *marginView;
+
+  CGFloat zoomDelta;
+
+  // Area to display additional controls (e.g. zoom info, caret position, status info).
+  NSView <InfoBarCommunicator>* mInfoBar;
+  BOOL mInfoBarAtTop;
+
+  id<ScintillaNotificationProtocol> mDelegate;
+}
 
 @synthesize backend = mBackend;
 @synthesize delegate = mDelegate;
@@ -1072,7 +1393,7 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
  * A delegate can be set to receive all notifications. If set no handling takes place here, except
  * for action pertaining to internal stuff (like the info bar).
  */
-- (void) notification: (Scintilla::SCNotification*)scn
+- (void) notification: (SCNotification*)scn
 {
   // Parent notification. Details are passed as SCNotification structure.
 
@@ -1135,6 +1456,21 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Setup a special indicator used in the editor to provide visual feedback for
+ * input composition, depending on language, keyboard etc.
+ */
+- (void) updateIndicatorIME
+{
+  [self setColorProperty: SCI_INDICSETFORE parameter: INDIC_IME fromHTML: @"#FF0000"];
+  const bool drawInBackground = [self message: SCI_GETPHASESDRAW] != 0;
+  [self setGeneralProperty: SCI_INDICSETUNDER parameter: INDIC_IME value: drawInBackground];
+  [self setGeneralProperty: SCI_INDICSETSTYLE parameter: INDIC_IME value: INDIC_PLAIN];
+  [self setGeneralProperty: SCI_INDICSETALPHA parameter: INDIC_IME value: 100];
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Initialization of the view. Used to setup a few other things we need.
  */
@@ -1168,18 +1504,13 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
     [scrollView setHasVerticalRuler:YES];
     [scrollView setRulersVisible:YES];
 
-    mBackend = new ScintillaCocoa(mContent, marginView);
+    mBackend = new ScintillaCocoa(self, mContent, marginView);
 
     // Establish a connection from the back end to this container so we can handle situations
     // which require our attention.
     mBackend->SetDelegate(self);
 
-    // Setup a special indicator used in the editor to provide visual feedback for
-    // input composition, depending on language, keyboard etc.
-    [self setColorProperty: SCI_INDICSETFORE parameter: INDIC_IME fromHTML: @"#FF0000"];
-    [self setGeneralProperty: SCI_INDICSETUNDER parameter: INDIC_IME value: 1];
-    [self setGeneralProperty: SCI_INDICSETSTYLE parameter: INDIC_IME value: INDIC_PLAIN];
-    [self setGeneralProperty: SCI_INDICSETALPHA parameter: INDIC_IME value: 100];
+    [self updateIndicatorIME];
 
     NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
     [center addObserver:self
@@ -1196,7 +1527,7 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
                selector:@selector(windowWillMove:)
                    name:NSWindowWillMoveNotification
                  object:[self window]];
-    
+
     [[scrollView contentView] setPostsBoundsChangedNotifications:YES];
     [center addObserver:self
 	       selector:@selector(scrollerAction:)
@@ -1212,6 +1543,10 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
 {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   delete mBackend;
+  mBackend = NULL;
+  mContent.owner = nil;
+  [marginView setClientView:nil];
+  [scrollView removeFromSuperview];
   [marginView release];
   [super dealloc];
 }
@@ -1310,6 +1645,7 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
  */
 - (void) scrollerAction: (id) sender
 {
+#pragma unused(sender)
   mBackend->UpdateForScroll();
 }
 
@@ -1695,7 +2031,7 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
 /**
  * Sets the notification callback
  */
-- (void) registerNotifyCallback: (intptr_t) windowid value: (Scintilla::SciNotifyFunc) callback
+- (void) registerNotifyCallback: (intptr_t) windowid value: (SciNotifyFunc) callback
 {
 	mBackend->RegisterNotifyCallback(windowid, callback);
 }
@@ -1744,6 +2080,21 @@ sourceOperationMaskForDraggingContext: (NSDraggingContext) context
   return [mContent selectedRange];
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Return the main selection as an NSRange of positions (not characters).
+ * Unlike selectedRange, this can return empty ranges inside the document.
+ */
+
+- (NSRange) selectedRangePositions
+{
+  const sptr_t positionBegin = [self message: SCI_GETSELECTIONSTART];
+  const sptr_t positionEnd = [self message: SCI_GETSELECTIONEND];
+  return NSMakeRange(positionBegin, positionEnd-positionBegin);
+}
+
+
 //--------------------------------------------------------------------------------------------------
 
 - (void)insertText: (id) aString
