@@ -9,6 +9,7 @@
 #include <time.h>
 
 #include <string>
+#include <sstream>
 
 #include <gtk/gtk.h>
 
@@ -17,12 +18,6 @@
 
 #include "GUI.h"
 
-#if GTK_CHECK_VERSION(2,20,0)
-#define IS_WIDGET_FOCUSSED(w) (gtk_widget_has_focus(GTK_WIDGET(w)))
-#else
-#define IS_WIDGET_FOCUSSED(w) (GTK_WIDGET_HAS_FOCUS(w))
-#endif
-
 namespace GUI {
 
 gui_string StringFromUTF8(const char *s) {
@@ -46,8 +41,20 @@ gui_string StringFromInteger(long i) {
 	return gui_string(number);
 }
 
+gui_string StringFromLongLong(long long i) {
+	try {
+		std::ostringstream strstrm;
+		strstrm << i;
+		return StringFromUTF8(strstrm.str());
+	} catch (std::exception &) {
+		// Exceptions not enabled on stream but still causes diagnostic in Coverity.
+		// Simply swallow the failure and return the default value.
+	}
+	return gui_string();
+}
+
 static GtkWidget *PWidget(WindowID wid) {
-	return reinterpret_cast<GtkWidget *>(wid);
+	return static_cast<GtkWidget *>(wid);
 }
 
 void Window::Destroy() {
@@ -57,7 +64,7 @@ void Window::Destroy() {
 }
 
 bool Window::HasFocus() {
-	return IS_WIDGET_FOCUSSED(wid);
+	return gtk_widget_has_focus(GTK_WIDGET(wid));
 }
 
 Rectangle Window::GetPosition() {
@@ -65,11 +72,7 @@ Rectangle Window::GetPosition() {
 	Rectangle rc(0, 0, 1000, 1000);
 	if (wid) {
 		GtkAllocation allocation;
-#if GTK_CHECK_VERSION(3,0,0)
 		gtk_widget_get_allocation(PWidget(wid), &allocation);
-#else
-		allocation = PWidget(wid)->allocation;
-#endif
 		rc.left = allocation.x;
 		rc.top = allocation.y;
 		if (allocation.width > 20) {
@@ -112,12 +115,7 @@ void Window::SetTitle(const char *s) {
 void Menu::CreatePopUp() {
 	Destroy();
 	mid = gtk_menu_new();
-#if GLIB_CHECK_VERSION(2,10,0)
-	 g_object_ref_sink(G_OBJECT(mid));
-#else
-	g_object_ref(G_OBJECT(mid));
-	gtk_object_sink(GTK_OBJECT(G_OBJECT(mid)));
-#endif
+	g_object_ref_sink(G_OBJECT(mid));
 }
 
 void Menu::Destroy() {
@@ -127,7 +125,7 @@ void Menu::Destroy() {
 }
 
 static void  MenuPositionFunc(GtkMenu *, gint *x, gint *y, gboolean *, gpointer userData) {
-	sptr_t intFromPointer = reinterpret_cast<sptr_t>(userData);
+	sptr_t intFromPointer = GPOINTER_TO_INT(userData);
 	*x = intFromPointer & 0xffff;
 	*y = intFromPointer >> 16;
 }
@@ -135,7 +133,7 @@ static void  MenuPositionFunc(GtkMenu *, gint *x, gint *y, gboolean *, gpointer
 void Menu::Show(Point pt, Window &) {
 	int screenHeight = gdk_screen_height();
 	int screenWidth = gdk_screen_width();
-	GtkMenu *widget = reinterpret_cast<GtkMenu *>(mid);
+	GtkMenu *widget = static_cast<GtkMenu *>(mid);
 	gtk_widget_show_all(GTK_WIDGET(widget));
 	GtkRequisition requisition;
 #if GTK_CHECK_VERSION(3,0,0)
@@ -150,7 +148,7 @@ void Menu::Show(Point pt, Window &) {
 		pt.y = screenHeight - requisition.height;
 	}
 	gtk_menu_popup(widget, NULL, NULL, MenuPositionFunc,
-		reinterpret_cast<void *>((pt.y << 16) | pt.x), 0,
+		GINT_TO_POINTER((pt.y << 16) | pt.x), 0,
 		gtk_get_current_event_time());
 }
 
@@ -180,10 +178,6 @@ sptr_t ScintillaWindow::Send(unsigned int msg, uptr_t wParam, sptr_t lParam) {
 	return scintilla_send_message(SCINTILLA(GetID()), msg, wParam, lParam);
 }
 
-sptr_t ScintillaWindow::SendPointer(unsigned int msg, uptr_t wParam, void *lParam) {
-	return scintilla_send_message(SCINTILLA(GetID()), msg, wParam, reinterpret_cast<sptr_t>(lParam));
-}
-
 bool IsDBCSLeadByte(int codePage, char ch) {
 	// Byte ranges found in Wikipedia articles with relevant search strings in each case
 	unsigned char uch = static_cast<unsigned char>(ch);
