@@ -36,7 +36,7 @@ GtkWidget* WBase::Pointer() {
 }
 
 bool WBase::Sensitive() {
-	return IS_WIDGET_SENSITIVE(Pointer());
+	return gtk_widget_get_sensitive(GTK_WIDGET(Pointer()));
 }
 
 void WStatic::Create(GUI::gui_string text) {
@@ -78,8 +78,9 @@ void WEntry::SetValid(GtkEntry *entry, bool valid) {
 	gtk_widget_set_name(GTK_WIDGET(entry), valid ? "" : "entryInvalid");
 #else
 	if (valid) {
-		GdkColor white = { 0, 0xFFFF, 0xFFFF, 0xFFFF};
-		gtk_widget_modify_base(GTK_WIDGET(entry), GTK_STATE_NORMAL, &white);
+		// Reset widget's background color
+		// to the one given by the GTK theme
+		gtk_widget_modify_base(GTK_WIDGET(entry), GTK_STATE_NORMAL, NULL);
 	} else {
 		GdkColor red = { 0, 0xFFFF, 0x6666, 0x6666 };
 		gtk_widget_modify_base(GTK_WIDGET(entry), GTK_STATE_NORMAL, &red);
@@ -112,7 +113,7 @@ void WComboBoxEntry::SetText(const GUI::gui_char *text) {
 }
 
 bool WComboBoxEntry::HasFocusOnSelfOrChild() {
-	return HasFocus() || IS_WIDGET_FOCUSSED(Entry());
+	return HasFocus() || gtk_widget_has_focus(GTK_WIDGET(Entry()));
 }
 
 void WComboBoxEntry::RemoveText(int position) {
@@ -159,21 +160,13 @@ void WComboBoxEntry::FillFromMemory(const std::vector<std::string> &mem, bool us
 
 void WButton::Create(GUI::gui_string text, GCallback func, gpointer data) {
 	SetID(gtk_button_new_with_mnemonic(text.c_str()));
-#if GTK_CHECK_VERSION(2,20,0)
 	gtk_widget_set_can_default(GTK_WIDGET(GetID()), TRUE);
-#else
-	GTK_WIDGET_SET_FLAGS(GetID(), GTK_CAN_DEFAULT);
-#endif
 	g_signal_connect(G_OBJECT(GetID()), "clicked", func, data);
 }
 
 void WButton::Create(GUI::gui_string text) {
 	SetID(gtk_button_new_with_mnemonic(text.c_str()));
-#if GTK_CHECK_VERSION(2,20,0)
 	gtk_widget_set_can_default(GTK_WIDGET(GetID()), TRUE);
-#else
-	GTK_WIDGET_SET_FLAGS(GetID(), GTK_CAN_DEFAULT);
-#endif
 }
 
 void WToggle::Create(const GUI::gui_string &text) {
@@ -190,21 +183,10 @@ void WProgress::Create() {
 	SetID(gtk_progress_bar_new());
 }
 
-WCheckDraw::WCheckDraw() : isActive(false), pbGrey(0), over(false), cdfn(NULL), user(NULL) {
-#if !GTK_CHECK_VERSION(3,4,0)
-	pStyle = 0;
-#endif
+WCheckDraw::WCheckDraw() : cdfn(NULL), user(NULL) {
 }
 
 WCheckDraw::~WCheckDraw() {
-	if (pbGrey)
-		g_object_unref(pbGrey);
-	pbGrey = 0;
-#if !GTK_CHECK_VERSION(3,4,0)
-	if (pStyle)
-		g_object_unref(pStyle);
-	pStyle = 0;
-#endif
 }
 
 static void GreyToAlpha(GdkPixbuf *ppb, GdkColor fore) {
@@ -226,72 +208,63 @@ static void GreyToAlpha(GdkPixbuf *ppb, GdkColor fore) {
 }
 
 void WCheckDraw::Create(const char **xpmImage, GUI::gui_string toolTip, GtkStyle *pStyle_) {
-	isActive = false;
-	pbGrey = gdk_pixbuf_new_from_xpm_data(xpmImage);
+	GtkWidget *button = gtk_toggle_button_new();
+	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
 
-	GtkWidget *da = gtk_drawing_area_new();
-#if GTK_CHECK_VERSION(3,4,0)
+	GdkPixbuf *pbGrey = gdk_pixbuf_new_from_xpm_data(xpmImage);
+	GdkPixbuf *pbAlpha = gdk_pixbuf_add_alpha(pbGrey, TRUE, 0xff, 0xff, 0);
+	g_object_unref(pbGrey);
+#if GTK_CHECK_VERSION(3, 4, 0)
 	(void)pStyle_;
+	GdkRGBA rgbaFore;
+	GtkStyleContext *context = gtk_widget_get_style_context(button);
+	gtk_style_context_get_color(context, gtk_style_context_get_state(context), &rgbaFore);
+	GdkColor fore;
+	fore.red = rgbaFore.red * 65535;
+	fore.green = rgbaFore.green * 65535;
+	fore.blue = rgbaFore.blue * 65535;
+	fore.pixel = 0;
 #else
-	pStyle = gtk_style_copy(pStyle_);
+	GdkColor fore = pStyle_->fg[gtk_widget_get_state(button)];
 #endif
+	// Convert the grey to alpha and make black
+	GreyToAlpha(pbAlpha, fore);
+	GtkWidget *image = gtk_image_new_from_pixbuf(pbAlpha);
+	g_object_unref(pbAlpha);
 
-#if GTK_CHECK_VERSION(2,20,0)
-	gtk_widget_set_can_focus(da, TRUE);
-	gtk_widget_set_sensitive(da, TRUE);
-#else
-	GTK_WIDGET_SET_FLAGS(da, GTK_CAN_FOCUS);
-	GTK_WIDGET_SET_FLAGS(da, GTK_SENSITIVE);
-#endif
+	SetID(button);
+	gtk_container_add(GTK_CONTAINER(button), image);
 
-	gtk_widget_set_events(da,
-			      GDK_EXPOSURE_MASK
-	                      | GDK_FOCUS_CHANGE_MASK
-			      | GDK_ENTER_NOTIFY_MASK
-			      | GDK_LEAVE_NOTIFY_MASK
-			      | GDK_BUTTON_PRESS_MASK
-			      | GDK_BUTTON_RELEASE_MASK
-			      | GDK_POINTER_MOTION_MASK
-			      | GDK_POINTER_MOTION_HINT_MASK
-			     );
-	gtk_widget_set_size_request(da, checkButtonWidth, 20);
-	SetID(da);
+	g_signal_connect(button, "toggled", G_CALLBACK(Toggled), this);
+	gtk_widget_set_name(GTK_WIDGET(button), "toggler");
 
 	GUI::gui_string toolTipNoMnemonic = toolTip;
 	size_t posMnemonic = toolTipNoMnemonic.find("_");
 	if (posMnemonic != GUI::gui_string::npos)
 		toolTipNoMnemonic.replace(posMnemonic, 1, "");
-#if GTK_CHECK_VERSION(2,12,0)
-	gtk_widget_set_tooltip_text(da, toolTipNoMnemonic.c_str());
-#endif
+	gtk_widget_set_tooltip_text(Pointer(), toolTipNoMnemonic.c_str());
+}
 
-	g_signal_connect(G_OBJECT(da), "focus-in-event", G_CALLBACK(Focus), this);
-	g_signal_connect(G_OBJECT(da), "focus-out-event", G_CALLBACK(Focus), this);
-	g_signal_connect(G_OBJECT(da), "button-press-event", G_CALLBACK(ButtonsPress), this);
-	g_signal_connect(G_OBJECT(da), "enter-notify-event", G_CALLBACK(MouseEnterLeave), this);
-	g_signal_connect(G_OBJECT(da), "leave-notify-event", G_CALLBACK(MouseEnterLeave), this);
-	g_signal_connect(G_OBJECT(da), "key-press-event", G_CALLBACK(KeyDown), this);
-#if GTK_CHECK_VERSION(3,0,0)
-	g_signal_connect(G_OBJECT(da), "draw", G_CALLBACK(DrawEvent), this);
-#else
-	g_signal_connect(G_OBJECT(da), "expose-event", G_CALLBACK(ExposeEvent), this);
-#endif
+void WCheckDraw::Toggled(GtkWidget *, WCheckDraw *pcd) {
+	if (pcd->cdfn)
+		pcd->cdfn(pcd, pcd->user);
+}
+
+GtkToggleButton *WCheckDraw::ToggleButton() {
+	return reinterpret_cast<GtkToggleButton*>(GetID());
 }
 
 bool WCheckDraw::Active() {
-	return isActive;
+	return gtk_toggle_button_get_active(ToggleButton());
 }
 
 void WCheckDraw::SetActive(bool active) {
-	isActive = active;
-	InvalidateAll();
+	gtk_toggle_button_set_active(ToggleButton(), active);
 }
 
 void WCheckDraw::Toggle() {
-	isActive = !isActive;
-	InvalidateAll();
-	if (cdfn)
-		cdfn(this, user);
+	SetActive(!Active());
 }
 
 void WCheckDraw::SetChangeFunction(ChangeFunction cdfn_, void *user_) {
@@ -299,201 +272,6 @@ void WCheckDraw::SetChangeFunction(ChangeFunction cdfn_, void *user_) {
 	user = user_;
 }
 
-gboolean WCheckDraw::Focus(GtkWidget */*widget*/, GdkEventFocus */*event*/, WCheckDraw *pcd) {
-	pcd->InvalidateAll();
-	return FALSE;
-}
-
-gboolean WCheckDraw::KeyDown(GtkWidget */*widget*/, GdkEventKey *event, WCheckDraw *pcd) {
-	if (event->keyval == ' ') {
-		pcd->Toggle();
-	}
-	return FALSE;
-}
-
-gint WCheckDraw::Press(GtkWidget *widget, GdkEventButton *event) {
-	if (event->button == 3) {
-		// Allow to go to container for pop up menu
-		return FALSE;
-	} else {
-		gtk_widget_grab_focus(widget);
-		Toggle();
-		return TRUE;
-	}
-}
-
-gint WCheckDraw::ButtonsPress(GtkWidget *widget, GdkEventButton *event, WCheckDraw *pcd) {
-	return pcd->Press(widget, event);
-}
-
-gboolean WCheckDraw::MouseEnterLeave(GtkWidget */*widget*/, GdkEventCrossing *event, WCheckDraw *pcd) {
-	pcd->over = event->type == GDK_ENTER_NOTIFY;
-	pcd->InvalidateAll();
-	return FALSE;
-}
-
-#if GTK_CHECK_VERSION(3,0,0)
-
-gboolean WCheckDraw::Draw(GtkWidget *widget, cairo_t *cr) {
-	GtkAllocation allocation;
-	gtk_widget_get_allocation(widget, &allocation);
-
-#if GTK_CHECK_VERSION(3,4,0)
-	GtkStyleContext *context = gtk_widget_get_style_context(widget);
-	gtk_style_context_save(context);
-	gtk_style_context_add_class(context, GTK_STYLE_CLASS_BUTTON);
-#else
-	GdkWindow *window = gtk_widget_get_window(widget);
-	pStyle = gtk_style_attach(pStyle, window);
-#endif
-
-	int heightOffset = (allocation.height - checkButtonWidth) / 2;
-	if (heightOffset < 0)
-		heightOffset = 0;
-
-	bool active = isActive;
-#if GTK_CHECK_VERSION(3,4,0)
-	GtkStateFlags flags = active ? GTK_STATE_FLAG_ACTIVE : GTK_STATE_FLAG_NORMAL;
-	if (over) {
-		flags = static_cast<GtkStateFlags>(flags | GTK_STATE_FLAG_PRELIGHT);
-	}
-#else
-	GtkStateType state = active ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
-	if (over) {
-		state = GTK_STATE_PRELIGHT;
-	}
-#endif
-	if (active || over) {
-#if GTK_CHECK_VERSION(3,4,0)
-		gtk_style_context_set_state(context, flags);
-		gtk_render_background(context, cr, 0, 0,
-			allocation.width, allocation.height);
-		gtk_render_frame(context, cr, 0, 0,
-			allocation.width, allocation.height);
-#else
-		GtkShadowType shadow = over ? GTK_SHADOW_OUT : GTK_SHADOW_IN;
-		gtk_paint_box(pStyle,
-			cr,
-			state,
-			shadow,
-			widget, const_cast<char *>("button"),
-			0, 0,
-			allocation.width, allocation.height);
-#endif
-	}
-
-	if (HasFocus()) {
-		// Draw focus inset by 2 pixels
-#if GTK_CHECK_VERSION(3,4,0)
-		gtk_render_focus(context, cr, 2, 2,
-			allocation.width-4, allocation.height-4);
-#else
-		gtk_paint_focus(pStyle,
-			cr,
-			state,
-			widget, const_cast<char *>("button"),
-			2, 2,
-			allocation.width-4, allocation.height-4);
-#endif
-	}
-
-#if GTK_CHECK_VERSION(3,4,0)
-	GdkRGBA rgbaFore;
-	gtk_style_context_get_color(context, GTK_STATE_FLAG_NORMAL, &rgbaFore);
-	GdkColor fore;
-	fore.red = rgbaFore.red * 65535;
-	fore.green = rgbaFore.green * 65535;
-	fore.blue = rgbaFore.blue * 65535;
-	fore.pixel = 0;
-	gtk_style_context_restore(context);
-#else
-	GdkColor fore = pStyle->fg[GTK_STATE_NORMAL];
-#endif
-	// Give it an alpha channel
-	GdkPixbuf *pbAlpha = gdk_pixbuf_add_alpha(pbGrey, TRUE, 0xff, 0xff, 0);
-	// Convert the grey to alpha and make black
-	GreyToAlpha(pbAlpha, fore);
-
-	int activeOffset = active ? 1 : 0;
-	int xOffset = 1 + 2 + activeOffset;
-	int yOffset = 3 + heightOffset + activeOffset;
-	gdk_cairo_set_source_pixbuf(cr, pbAlpha, xOffset, yOffset);
-	cairo_rectangle(cr,
-		xOffset, yOffset,
-		checkIconWidth, checkIconWidth);
-	cairo_fill(cr);
-	g_object_unref(pbAlpha);
-
-	return TRUE;
-}
-
-gboolean WCheckDraw::DrawEvent(GtkWidget *widget, cairo_t *cr, WCheckDraw *pcd) {
-	return pcd->Draw(widget, cr);
-}
-
-#else
-
-gboolean WCheckDraw::Expose(GtkWidget *widget, GdkEventExpose */*event*/) {
-	pStyle = gtk_style_attach(pStyle, widget->window);
-
-	GdkRectangle area;
-	area.x = 0;
-	area.y = 0;
-	area.width = widget->allocation.width;
-	area.height = widget->allocation.height;
-	int heightOffset = (area.height - checkButtonWidth) / 2;
-	if (heightOffset < 0)
-		heightOffset = 0;
-	GdkGC *gcDraw = gdk_gc_new(GDK_DRAWABLE(widget->window));
-	bool active = isActive;
-	GtkStateType state = active ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
-	GtkShadowType shadow = GTK_SHADOW_IN;
-	if (over) {
-		state = GTK_STATE_PRELIGHT;
-		shadow = GTK_SHADOW_OUT;
-	}
-	if (active || over)
-		gtk_paint_box(pStyle, widget->window,
-			       state,
-			       shadow,
-			       &area, widget, const_cast<char *>("button"),
-			       0, 0,
-			       area.width, area.height);
-	if (HasFocus()) {
-		// Draw focus inset by 2 pixels
-		gtk_paint_focus(pStyle, widget->window,
-			       state,
-			       &area, widget, const_cast<char *>("button"),
-			       2, 2,
-			       area.width-4, area.height-4);
-	}
-
-	GdkColor fore = pStyle->fg[GTK_STATE_NORMAL];
-	// Give it an alpha channel
-	GdkPixbuf *pbAlpha = gdk_pixbuf_add_alpha(pbGrey, TRUE, 0xff, 0xff, 0);
-	// Convert the grey to alpha and make black
-	GreyToAlpha(pbAlpha, fore);
-
-	int activeOffset = active ? 1 : 0;
-	gdk_draw_pixbuf(
-		widget->window,
-		gcDraw,
-		pbAlpha,
-		0, 0,
-		1 + 2 + activeOffset, 3 + heightOffset + activeOffset,
-		checkIconWidth, checkIconWidth,
-		GDK_RGB_DITHER_NONE, 0, 0);
-	g_object_unref(pbAlpha);
-	g_object_unref(gcDraw);
-	return TRUE;
-}
-
-gboolean WCheckDraw::ExposeEvent(GtkWidget *widget, GdkEventExpose *event, WCheckDraw *pcd) {
-	return pcd->Expose(widget, event);
-}
-
-#endif
-
 #if GTK_CHECK_VERSION(3,4,0)
 #define USE_GRID 1
 #else
@@ -673,7 +451,7 @@ bool Strip::KeyDown(GdkEventKey *event) {
 }
 
 void Strip::MenuSignal(GtkMenuItem *menuItem, Strip *pStrip) {
-	sptr_t cmd = (sptr_t)(g_object_get_data(G_OBJECT(menuItem), "CmdNum"));
+	sptr_t cmd = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuItem), "CmdNum"));
 	pStrip->MenuAction(cmd);
 }
 
@@ -682,7 +460,7 @@ void Strip::AddToPopUp(GUI::Menu &popup, const char *label, int cmd, bool checke
 	GUI::gui_string localised = localiser->Text(label);
 	GtkWidget *menuItem = gtk_check_menu_item_new_with_mnemonic(localised.c_str());
 	gtk_menu_shell_append(GTK_MENU_SHELL(popup.GetID()), menuItem);
-	g_object_set_data(G_OBJECT(menuItem), "CmdNum", reinterpret_cast<void *>((sptr_t)(cmd)));
+	g_object_set_data(G_OBJECT(menuItem), "CmdNum", GINT_TO_POINTER(cmd));
 	g_signal_connect(G_OBJECT(menuItem),"activate", G_CALLBACK(MenuSignal), this);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItem), checked ? TRUE : FALSE);
 	allowMenuActions = true;
