@@ -5,14 +5,6 @@
 
 // Callback thunk class connects GTK+ signals to a simple command method.
 
-#if GTK_CHECK_VERSION(2,20,0)
-#define IS_WIDGET_FOCUSSED(w) (gtk_widget_has_focus(GTK_WIDGET(w)))
-#define IS_WIDGET_SENSITIVE(w) (gtk_widget_get_sensitive(GTK_WIDGET(w)))
-#else
-#define IS_WIDGET_FOCUSSED(w) (GTK_WIDGET_HAS_FOCUS(w))
-#define IS_WIDGET_SENSITIVE(w) (GTK_WIDGET_SENSITIVE(w))
-#endif
-
 // Callback thunk class connects GTK+ signals to an instance method.
 template< class T, void (T::*method)() >
 class ObjectSignal {
@@ -30,7 +22,7 @@ public:
 };
 
 inline GtkWidget *Widget(const GUI::Window &w) {
-	return reinterpret_cast<GtkWidget *>(w.GetID());
+	return static_cast<GtkWidget *>(w.GetID());
 }
 
 class WStatic : public WBase {
@@ -81,26 +73,10 @@ class WCheckDraw : public WBase {
 public:
 	typedef void (*ChangeFunction)(WCheckDraw *cd, void *user);
 private:
-	bool isActive;
-	GdkPixbuf *pbGrey;
-#if !GTK_CHECK_VERSION(3,4,0)
-	GtkStyle *pStyle;
-#endif
-	bool over;
 	ChangeFunction cdfn;
 	void *user;
-	static gboolean Focus(GtkWidget *widget, GdkEventFocus *event, WCheckDraw *pcd);
-	gint Press(GtkWidget *widget, GdkEventButton *event);
-	static gint ButtonsPress(GtkWidget *widget, GdkEventButton *event, WCheckDraw *pcd);
-	static gboolean MouseEnterLeave(GtkWidget *widget, GdkEventCrossing *event, WCheckDraw *pcd);
-	static gboolean KeyDown(GtkWidget *widget, GdkEventKey *event, WCheckDraw *pcd);
-#if GTK_CHECK_VERSION(3,0,0)
-	gboolean Draw(GtkWidget *widget, cairo_t *cr);
-	static gboolean DrawEvent(GtkWidget *widget, cairo_t *cr, WCheckDraw *pcd);
-#else
-	gboolean Expose(GtkWidget *widget, GdkEventExpose *event);
-	static gboolean ExposeEvent(GtkWidget *widget, GdkEventExpose *event, WCheckDraw *pcd);
-#endif
+	static void Toggled(GtkWidget *widget, WCheckDraw *pcd);
+	GtkToggleButton *ToggleButton();
 public:
 	WCheckDraw();
 	~WCheckDraw();
