@@ -29,6 +29,17 @@
 #define SC_INDICATOR_CONVERTED INDIC_IME+2
 #define SC_INDICATOR_UNKNOWN INDIC_IME_MAX
 
+// Q_WS_MAC and Q_WS_X11 aren't defined in Qt5
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+#ifdef Q_OS_MAC
+#define Q_WS_MAC 1
+#endif
+
+#if !defined(Q_OS_MAC) && !defined(Q_OS_WIN)
+#define Q_WS_X11 1
+#endif
+#endif // QT_VERSION >= 5.0.0
+
 #ifdef SCI_NAMESPACE
 using namespace Scintilla;
 #endif
@@ -47,7 +58,7 @@ ScintillaEditBase::ScintillaEditBase(QWidget *parent)
 	setFrameStyle(QFrame::NoFrame);
 	setFocusPolicy(Qt::StrongFocus);
 	setAttribute(Qt::WA_StaticContents);
-	viewport()->setAttribute(Qt::WA_OpaquePaintEvent);
+	viewport()->setAutoFillBackground(false);
 	setAttribute(Qt::WA_KeyCompression);
 	setAttribute(Qt::WA_InputMethodEnabled);
 
@@ -289,9 +300,7 @@ void ScintillaEditBase::mousePressEvent(QMouseEvent *event)
 		return;
 	}
 
-	bool button = event->button() == Qt::LeftButton;
-
-	if (button) {
+	if (event->button() == Qt::LeftButton) {
 		bool shift = QApplication::keyboardModifiers() & Qt::ShiftModifier;
 		bool ctrl  = QApplication::keyboardModifiers() & Qt::ControlModifier;
 #ifdef Q_WS_X11
@@ -304,6 +313,14 @@ void ScintillaEditBase::mousePressEvent(QMouseEvent *event)
 
 		sqt->ButtonDown(pos, time.elapsed(), shift, ctrl, alt);
 	}
+
+	if (event->button() == Qt::RightButton) {
+		bool shift = QApplication::keyboardModifiers() & Qt::ShiftModifier;
+		bool ctrl  = QApplication::keyboardModifiers() & Qt::ControlModifier;
+		bool alt   = QApplication::keyboardModifiers() & Qt::AltModifier;
+
+		sqt->RightButtonDownWithModifiers(pos, time.elapsed(), ScintillaQt::ModifierFlags(shift, ctrl, alt));
+	}
 }
 
 void ScintillaEditBase::mouseReleaseEvent(QMouseEvent *event)
@@ -350,9 +367,12 @@ void ScintillaEditBase::contextMenuEvent(QContextMenuEvent *event)
 {
 	Point pos = PointFromQPoint(event->globalPos());
 	Point pt = PointFromQPoint(event->pos());
-	if (!sqt->PointInSelection(pt))
+	if (!sqt->PointInSelection(pt)) {
 		sqt->SetEmptySelection(sqt->PositionFromLocation(pt));
-	sqt->ContextMenu(pos);
+	}
+	if (sqt->ShouldDisplayPopup(pt)) {
+		sqt->ContextMenu(pos);
+	}
 }
 
 void ScintillaEditBase::dragEnterEvent(QDragEnterEvent *event)
@@ -409,10 +429,10 @@ bool ScintillaEditBase::IsHangul(const QChar qchar)
 	const bool HangulJamoExtendedB = (0xD7B0 <= unicode && unicode <= 0xD7FF);
 	const bool HangulSyllable = (0xAC00 <= unicode && unicode <= 0xD7A3);
 	return HangulJamo || HangulCompatibleJamo  || HangulSyllable ||
-				HangulJamoExtendedA || HangulJamoExtendedB; 
+				HangulJamoExtendedA || HangulJamoExtendedB;
 }
 
-void ScintillaEditBase::MoveImeCarets(int offset) 
+void ScintillaEditBase::MoveImeCarets(int offset)
 {
 	// Move carets relatively by bytes
 	for (size_t r=0; r < sqt->sel.Count(); r++) {
@@ -421,8 +441,8 @@ void ScintillaEditBase::MoveImeCarets(int offset)
 		sqt->sel.Range(r).anchor.SetPosition(positionInsert + offset);
  	}
 }
- 
-void ScintillaEditBase::DrawImeIndicator(int indicator, int len) 
+
+void ScintillaEditBase::DrawImeIndicator(int indicator, int len)
 {
 	// Emulate the visual style of IME characters with indicators.
 	// Draw an indicator on the character before caret by the character bytes of len
@@ -438,17 +458,79 @@ void ScintillaEditBase::DrawImeIndicator(int indicator, int len)
 	}
 }
 
+static int GetImeCaretPos(QInputMethodEvent *event)
+{
+	foreach (QInputMethodEvent::Attribute attr, event->attributes()) {
+		if (attr.type == QInputMethodEvent::Cursor)
+			return attr.start;
+	}
+	return 0;
+}
+
+static std::vector<int> MapImeIndicators(QInputMethodEvent *event)
+{
+	std::vector<int> imeIndicator(event->preeditString().size(), SC_INDICATOR_UNKNOWN);
+	foreach (QInputMethodEvent::Attribute attr, event->attributes()) {
+		if (attr.type == QInputMethodEvent::TextFormat) {
+			QTextFormat format = attr.value.value<QTextFormat>();
+			QTextCharFormat charFormat = format.toCharFormat();
+
+			int indicator = SC_INDICATOR_UNKNOWN;
+			switch (charFormat.underlineStyle()) {
+				case QTextCharFormat::NoUnderline: // win32, linux
+					indicator = SC_INDICATOR_TARGET;
+					break;
+				case QTextCharFormat::SingleUnderline: // osx
+				case QTextCharFormat::DashUnderline: // win32, linux
+					indicator = SC_INDICATOR_INPUT;
+					break;
+				case QTextCharFormat::DotLine:
+				case QTextCharFormat::DashDotLine:
+				case QTextCharFormat::WaveUnderline:
+				case QTextCharFormat::SpellCheckUnderline:
+					indicator = SC_INDICATOR_CONVERTED;
+					break;
+
+				default:
+					indicator = SC_INDICATOR_UNKNOWN;
+			}
+
+			if (format.hasProperty(QTextFormat::BackgroundBrush)) // win32, linux
+				indicator = SC_INDICATOR_TARGET;
+
+#ifdef Q_OS_OSX
+			if (charFormat.underlineStyle() == QTextCharFormat::SingleUnderline) {
+				QColor uc = charFormat.underlineColor();
+				if (uc.lightness() < 2) { // osx
+					indicator = SC_INDICATOR_TARGET;
+				}
+			}
+#endif
+
+			for (int i = attr.start; i < attr.start+attr.length; i++) {
+				imeIndicator[i] = indicator;
+			}
+		}
+	}
+	return imeIndicator;
+}
+
 void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 {
 	// Copy & paste by johnsonj with a lot of helps of Neil
 	// Great thanks for my forerunners, jiniya and BLUEnLIVE
 
+	if (sqt->pdoc->IsReadOnly() || sqt->SelectionContainsProtected()) {
+		// Here, a canceling and/or completing composition function is needed.
+		return;
+	}
+
 	if (sqt->pdoc->TentativeActive()) {
 		sqt->pdoc->TentativeUndo();
 	} else {
 		// No tentative undo means start of this composition so
 		// Fill in any virtual spaces.
-		sqt->FillVirtualSpace();
+		sqt->ClearBeforeTentativeStart();
 	}
 
 	sqt->view.imeCaretBlockOverride = false;
@@ -477,58 +559,7 @@ void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 
 		sqt->pdoc->TentativeStart(); // TentativeActive() from now on.
 
-		// Mark segments and get ime caret position.
-		unsigned int imeCaretPos = 0;
-		unsigned int imeIndicator[MAXLENINPUTIME] = {0};
-#ifdef Q_OS_LINUX
-		// ibus-qt has a bug to return only one underline style.
-		// Q_OS_LINUX blocks are temporary work around to cope with it.
-		unsigned int attrSegment = 0;
-#endif
-
-		foreach (QInputMethodEvent::Attribute attr, event->attributes()) {
-			if (attr.type == QInputMethodEvent::TextFormat) {
-				QTextFormat format = attr.value.value<QTextFormat>();
-				QTextCharFormat charFormat = format.toCharFormat();
-
-				unsigned int indicator = SC_INDICATOR_UNKNOWN;
-				switch (charFormat.underlineStyle()) {
-					case QTextCharFormat::NoUnderline:
-						indicator = SC_INDICATOR_TARGET; //target input
-						break;
-					case QTextCharFormat::SingleUnderline:
-					case QTextCharFormat::DashUnderline:
-						indicator = SC_INDICATOR_INPUT; //normal input
-						break;
-					case QTextCharFormat::DotLine:
-					case QTextCharFormat::DashDotLine:
-					case QTextCharFormat::WaveUnderline:
-					case QTextCharFormat::SpellCheckUnderline:
-						indicator = SC_INDICATOR_CONVERTED;
-						break;
-		
-					default:
-						indicator = SC_INDICATOR_UNKNOWN;
-				}
-
-#ifdef Q_OS_LINUX
-				attrSegment++;
-				indicator = attr.start;
-#endif
-				for (int i = attr.start; i < attr.start+attr.length; i++) {
-					imeIndicator[i] = indicator;
-				}
-			} else if (attr.type == QInputMethodEvent::Cursor) {
-				imeCaretPos = attr.start;
-			}
-		}
-#ifdef Q_OS_LINUX
-		const bool targetInput = (attrSegment > 1) || ((imeCaretPos == 0) && (preeditStr != preeditString));
-		preeditString = preeditStr;
-#endif
-		// Display preedit characters one by one.
-		int imeCharPos[MAXLENINPUTIME] = {0};
-		int numBytes = 0;
+		std::vector<int> imeIndicator = MapImeIndicators(event);
 
 		const bool recording = sqt->recordingMacro;
 		sqt->recordingMacro = false;
@@ -538,32 +569,28 @@ void ScintillaEditBase::inputMethodEvent(QInputMethodEvent *event)
 			const QByteArray oneChar = sqt->BytesForDocument(oneCharUTF16);
 			const int oneCharLen = oneChar.length();
 
-			// Record character positions for moving ime caret.
-			numBytes += oneCharLen;
-			imeCharPos[i+1] = numBytes;
-
 			sqt->AddCharUTF(oneChar.data(), oneCharLen);
 
-#ifdef Q_OS_LINUX
-			// Segment marked with imeCaretPos is for target input.
-			if ((imeIndicator[i] == imeCaretPos) && (targetInput)) {
-				DrawImeIndicator(SC_INDICATOR_TARGET, oneCharLen);
-			} else {
-				DrawImeIndicator(SC_INDICATOR_INPUT, oneCharLen);
-			}
-#else
 			DrawImeIndicator(imeIndicator[i], oneCharLen);
-#endif
 			i += ucWidth;
-		} 
+		}
 		sqt->recordingMacro = recording;
 
 		// Move IME carets.
+		int imeCaretPos = GetImeCaretPos(event);
+		int imeEndToImeCaretU16 = imeCaretPos - preeditStrLen;
+		int imeCaretPosDoc = sqt->pdoc->GetRelativePositionUTF16(sqt->CurrentPosition(), imeEndToImeCaretU16);
+
+		MoveImeCarets(- sqt->CurrentPosition() + imeCaretPosDoc);
+
 		if (IsHangul(preeditStr.at(0))) {
+#ifndef Q_OS_WIN
+			if (imeCaretPos > 0) {
+				int oneCharBefore = sqt->pdoc->GetRelativePosition(sqt->CurrentPosition(), -1);
+				MoveImeCarets(- sqt->CurrentPosition() + oneCharBefore);
+			}
+#endif
 			sqt->view.imeCaretBlockOverride = true;
-			MoveImeCarets(- imeCharPos[preeditStrLen]);
-		} else {
-			MoveImeCarets(- imeCharPos[preeditStrLen] + imeCharPos[imeCaretPos]);
 		}
 
 		// Set candidate box position for Qt::ImMicroFocus.
