@@ -80,7 +80,7 @@ static size_t FindCaseInsensitive(const std::vector<std::string> &values, const
 }
 
 // extract the next RTF control word from *style
-void GetRTFNextControl(const char **style, char *control) {
+static void GetRTFNextControl(const char **style, char *control) {
 	ptrdiff_t len;
 	const char *pos = *style;
 	*control = '\0';
@@ -94,7 +94,7 @@ void GetRTFNextControl(const char **style, char *control) {
 }
 
 // extracts control words that are different between two styles
-std::string GetRTFStyleChange(const char *last, const char *current) { // \f0\fs20\cf0\highlight0\b0\i0
+static std::string GetRTFStyleChange(const char *last, const char *current) { // \f0\fs20\cf0\highlight0\b0\i0
 	char lastControl[MAX_STYLEDEF], currentControl[MAX_STYLEDEF];
 	const char *lastPos = last;
 	const char *currentPos = current;
@@ -256,11 +256,11 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 		} else if (ch == '\r') {
 			os << RTF_EOLN;
 			column = -1;
-		} else if (isUTF8 && !isascii(ch)) {
+		} else if (isUTF8 && !IsASCII(ch)) {
 			const int nextPosition = wEditor.Call(SCI_POSITIONAFTER, iPos);
 			wEditor.Call(SCI_SETTARGETRANGE, iPos, nextPosition);
 			unsigned char u8Char[5] = "";
-			wEditor.CallString(SCI_TARGETASUTF8, 0, reinterpret_cast<char *>(u8Char));
+			wEditor.CallPointer(SCI_TARGETASUTF8, 0, u8Char);
 			unsigned int u32 = UTF32Character(u8Char);
 			if (u32 < 0x10000) {
 				os << "\\u" << static_cast<short>(u32) << "?";
@@ -278,17 +278,22 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 	os << RTF_BODYCLOSE;
 }
 
-void SciTEBase::SaveToRTF(FilePath saveName, int start, int end) {
+void SciTEBase::SaveToRTF(const FilePath &saveName, int start, int end) {
 	FILE *fp = saveName.Open(GUI_TEXT("wt"));
+	bool failedWrite = fp == NULL;
 	if (fp) {
 		std::ostringstream oss;
 		SaveToStreamRTF(oss, start, end);
 		std::string rtf = oss.str();
-		fwrite(rtf.c_str(), 1, rtf.length(), fp);
-		fclose(fp);
-	} else {
-		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", filePath.AsInternal());
-		WindowMessageBox(wSciTE, msg);
+		if (fwrite(rtf.c_str(), 1, rtf.length(), fp) != rtf.length()) {
+			failedWrite = true;
+		}
+		if (fclose(fp) != 0) {
+			failedWrite = true;
+		}
+	}
+	if (failedWrite) {
+		FailedSaveMessageBox(saveName);
 	}
 }
 
