@@ -71,6 +71,7 @@ gui_string StringFromUTF8(const char *s);
 gui_string StringFromUTF8(const std::string &s);
 std::string UTF8FromString(const gui_string &s);
 gui_string StringFromInteger(long i);
+gui_string StringFromLongLong(long long i);
 gui_string HexStringFromInteger(long i);
 
 typedef void *WindowID;
@@ -132,9 +133,9 @@ struct ScintillaFailure {
 };
 
 class ScintillaWindow : public Window {
-	// Private so ScintillaWindow objects can not be copied
-	ScintillaWindow(const ScintillaWindow &source);
-	ScintillaWindow &operator=(const ScintillaWindow &);
+	// Deleted so ScintillaWindow objects can not be copied
+	ScintillaWindow(const ScintillaWindow &source) = delete;
+	ScintillaWindow &operator=(const ScintillaWindow &) = delete;
 	SciFnDirect fn;
 	sptr_t ptr;
 public:
@@ -180,27 +181,18 @@ public:
 			throw ScintillaFailure(status);
 		return retVal;
 	}
+	int CallPointer(unsigned int msg, uptr_t wParam, void *s) {
+		return Call(msg, wParam, reinterpret_cast<sptr_t>(s));
+	}
 	int CallString(unsigned int msg, uptr_t wParam, const char *s) {
 		return Call(msg, wParam, reinterpret_cast<sptr_t>(s));
 	}
+	// Send is the basic method and can be used between threads on Win32
 	sptr_t Send(unsigned int msg, uptr_t wParam=0, sptr_t lParam=0);
-	sptr_t SendPointer(unsigned int msg, uptr_t wParam=0, void *lParam=0);
 };
 
 bool IsDBCSLeadByte(int codePage, char ch);
 
 }
 
-#if defined(SCI_NAMESPACE)
-
-// Scintilla namespace may or may not be turned on.
-// If it is turned on, then make the structures usable without the Scintilla:: prefix
-
-using Scintilla::Sci_CharacterRange;
-using Scintilla::Sci_TextRange;
-using Scintilla::Sci_TextToFind;
-using Scintilla::SCNotification;
-
-#endif
-
 #endif
