@@ -920,7 +920,20 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	// - numeric return type gets returned to lua as a number (following the stringresult)
 	// - other return types e.g. void get dropped.
 
-	sptr_t result = host->Send(p, func.value, params[0], params[1]);
+	sptr_t result = 0;
+	try {
+		result = host->Send(p, func.value, params[0], params[1]);
+	} catch (GUI::ScintillaFailure &sf) {
+		std::string failureExplanation;
+		failureExplanation += ">Lua: Scintilla failure ";
+		failureExplanation += StdStringFromInteger(static_cast<int>(sf.status));
+		failureExplanation += " for message ";
+		failureExplanation += StdStringFromInteger(func.value);
+		failureExplanation += ".\n";
+		// Reset status before continuing
+		host->Send(p, SCI_SETSTATUS, SC_STATUS_OK, 0);
+		host->Trace(failureExplanation.c_str());
+	}
 
 	int resultCount = 0;
 
