@@ -87,25 +87,22 @@ void Buffer::CancelLoad() {
 	}
 }
 
-BufferList::BufferList() : current(0), stackcurrent(0), stack(0), buffers(0), size(0), length(0), lengthVisible(0), initialised(false) {}
+BufferList::BufferList() : current(0), stackcurrent(0), stack(0), buffers(0), length(0), lengthVisible(0), initialised(false) {}
 
 BufferList::~BufferList() {
-	delete []buffers;
-	delete []stack;
 }
 
 void BufferList::Allocate(int maxSize) {
 	length = 1;
 	lengthVisible = 1;
 	current = 0;
-	size = maxSize;
-	buffers = new Buffer[size];
-	stack = new int[size];
+	buffers.resize(maxSize);
+	stack.resize(maxSize);
 	stack[0] = 0;
 }
 
 int BufferList::Add() {
-	if (length < size) {
+	if (length < size()) {
 		length++;
 	}
 	buffers[length - 1].Init();
@@ -125,7 +122,7 @@ int BufferList::GetDocumentByWorker(FileWorker *pFileWorker) const {
 	return -1;
 }
 
-int BufferList::GetDocumentByName(FilePath filename, bool excludeCurrent) {
+int BufferList::GetDocumentByName(const FilePath &filename, bool excludeCurrent) {
 	if (!filename.IsSet()) {
 		return -1;
 	}
@@ -182,7 +179,11 @@ int BufferList::Current() const {
 	return current;
 }
 
-Buffer *BufferList::CurrentBuffer() const {
+Buffer *BufferList::CurrentBuffer() {
+	return &buffers[Current()];
+}
+
+const Buffer *BufferList::CurrentBufferConst() const {
 	return &buffers[Current()];
 }
 
@@ -274,7 +275,7 @@ void BufferList::Swap(int indexA, int indexB) {
 }
 
 bool BufferList::SingleBuffer() const {
-	return size == 1;
+	return size() == 1;
 }
 
 BackgroundActivities BufferList::CountBackgroundActivities() const {
@@ -350,7 +351,7 @@ void BufferList::FinishedFuture(int index, Buffer::FutureDo fd) {
 }
 
 sptr_t SciTEBase::GetDocumentAt(int index) {
-	if (index < 0 || index >= buffers.size) {
+	if (index < 0 || index >= buffers.size()) {
 		return 0;
 	}
 	if (buffers.buffers[index].doc == 0) {
@@ -361,7 +362,7 @@ sptr_t SciTEBase::GetDocumentAt(int index) {
 }
 
 void SciTEBase::SwitchDocumentAt(int index, sptr_t pdoc) {
-	if (index < 0 || index >= buffers.size) {
+	if (index < 0 || index >= buffers.size()) {
 		return;
 	}
 	sptr_t pdocOld = buffers.buffers[index].doc;
@@ -392,7 +393,7 @@ void SciTEBase::SetDocumentAt(int index, bool updateStack) {
 	}
 
 	if (extender) {
-		if (buffers.size > 1)
+		if (buffers.size() > 1)
 			extender->ActivateBuffer(index);
 		else
 			extender->InitBuffer(0);
@@ -471,7 +472,7 @@ void SciTEBase::UpdateBuffersCurrent() {
 }
 
 bool SciTEBase::IsBufferAvailable() const {
-	return buffers.size > 1 && buffers.length < buffers.size;
+	return buffers.size() > 1 && buffers.length < buffers.size();
 }
 
 bool SciTEBase::CanMakeRoom(bool maySaveIfDirty) {
@@ -516,7 +517,7 @@ void SciTEBase::InitialiseBuffers() {
 		// First document is the default from creation of control
 		buffers.buffers[0].doc = wEditor.CallReturnPointer(SCI_GETDOCPOINTER, 0, 0);
 		wEditor.Call(SCI_ADDREFDOCUMENT, 0, buffers.buffers[0].doc); // We own this reference
-		if (buffers.size == 1) {
+		if (buffers.size() == 1) {
 			// Single buffer mode, delete the Buffers main menu entry
 			DestroyMenuItem(menuBuffers, 0);
 			// Destroy command "View Tab Bar" in the menu "View"
@@ -789,7 +790,9 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 		}
 	}
 
-	fclose(sessionFile);
+	if (fclose(sessionFile) != 0) {
+		FailedSaveMessageBox(sessionPathName);
+	}
 
 	FilePath sessionFilePath = FilePath(sessionPathName).AbsolutePath();
 	// Add/update SessionPath environment variable
@@ -843,7 +846,7 @@ void SciTEBase::New() {
 
 	propsDiscovered.Clear();
 
-	if ((buffers.size == 1) && (!buffers.buffers[0].IsUntitled())) {
+	if ((buffers.size() == 1) && (!buffers.buffers[0].IsUntitled())) {
 		AddFileToStack(buffers.buffers[0],
 		        buffers.buffers[0].selection,
 		        buffers.buffers[0].scrollPosition);
@@ -855,7 +858,7 @@ void SciTEBase::New() {
 	        (buffers.Current() != 0) ||
 	        (buffers.buffers[0].isDirty) ||
 	        (!buffers.buffers[0].IsUntitled())) {
-		if (buffers.size == buffers.length) {
+		if (buffers.size() == buffers.length) {
 			Close(false, false, true);
 		}
 		buffers.SetCurrent(buffers.Add());
@@ -917,7 +920,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		extender->OnClose(filePath.AsUTF8().c_str());
 	}
 
-	if (buffers.size == 1) {
+	if (buffers.size() == 1) {
 		// With no buffer list, Close means close from MRU
 		closingLast = !(recentFileStack[0].IsSet());
 		buffers.buffers[0].Init();
@@ -925,7 +928,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		ClearDocument(); //avoid double are-you-sure
 		if (!makingRoomForNew)
 			StackMenu(0); // calls New, or Open, which calls InitBuffer
-	} else if (buffers.size > 1) {
+	} else if (buffers.size() > 1) {
 		if (buffers.Current() >= 0 && buffers.Current() < buffers.length) {
 			UpdateBuffersCurrent();
 			Buffer buff = buffers.buffers[buffers.Current()];
@@ -1124,7 +1127,7 @@ static void EscapeFilePathsForMenu(GUI::gui_string &path) {
 }
 
 void SciTEBase::SetBuffersMenu() {
-	if (buffers.size <= 1) {
+	if (buffers.size() <= 1) {
         DestroyMenuItem(menuBuffers, IDM_BUFFERSEP);
     }
 	RemoveAllTabs();
@@ -1133,7 +1136,7 @@ void SciTEBase::SetBuffersMenu() {
 	for (pos = buffers.lengthVisible; pos < bufferMax; pos++) {
 		DestroyMenuItem(menuBuffers, IDM_BUFFER + pos);
 	}
-	if (buffers.size > 1) {
+	if (buffers.size() > 1) {
 		int menuStart = 4;
 		SetMenuItem(menuBuffers, menuStart, IDM_BUFFERSEP, GUI_TEXT(""));
 		for (pos = 0; pos < buffers.lengthVisible; pos++) {
@@ -1175,6 +1178,11 @@ void SciTEBase::SetBuffersMenu() {
 			//char *cpDirEnd = strrchr(buffers.buffers[pos]->fileName, pathSepChar);
 			//strcat(entry, cpDirEnd + 1);
 
+			if (buffers.buffers[pos].isReadOnly && props.GetInt("read.only.indicator"))  {
+				entry=entry+GUI_TEXT(" |");
+				titleTab=titleTab+GUI_TEXT(" |");
+			}
+
 			if (buffers.buffers[pos].isDirty) {
 				entry += GUI_TEXT(" *");
 				titleTab += GUI_TEXT(" *");
@@ -1256,7 +1264,7 @@ bool SciTEBase::AddFileToBuffer(const BufferState &bufferState) {
 	return opened;
 }
 
-void SciTEBase::AddFileToStack(FilePath file, SelectedRange selection, int scrollPos) {
+void SciTEBase::AddFileToStack(const FilePath &file, SelectedRange selection, int scrollPos) {
 	if (!file.IsSet())
 		return;
 	DeleteFileStackMenu();
@@ -1276,7 +1284,7 @@ void SciTEBase::AddFileToStack(FilePath file, SelectedRange selection, int scrol
 	SetFileStackMenu();
 }
 
-void SciTEBase::RemoveFileFromStack(FilePath file) {
+void SciTEBase::RemoveFileFromStack(const FilePath &file) {
 	if (!file.IsSet())
 		return;
 	DeleteFileStackMenu();
@@ -1490,7 +1498,7 @@ inline bool isdigitchar(int ch) {
 	return (ch >= '0') && (ch <= '9');
 }
 
-int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &column) {
+static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &column) {
 	sourcePath.clear();
 	column = -1; // default to not detected
 	switch (format) {
@@ -1527,14 +1535,15 @@ int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &co
 				++cdoc;
 			for (int i = 0; cdoc[i]; i++) {
 				if (cdoc[i] == ':' && isdigitchar(cdoc[i + 1])) {
-					int sourceNumber = atoi(cdoc + i + 1) - 1;
+					const int sourceLine = atoi(cdoc + i + 1);
 					sourcePath.assign(cdoc, i);
 					i += 2;
 					while (isdigitchar(cdoc[i]))
 						++i;
 					if (cdoc[i] == ':' && isdigitchar(cdoc[i + 1]))
 						column = atoi(cdoc + i + 1) - 1;
-					return sourceNumber;
+					// Some tools show whole file errors as occurring at line 0
+					return (sourceLine > 0) ? sourceLine - 1 : 0;
 				}
 			}
 			break;
@@ -1832,6 +1841,23 @@ int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &co
 	return -1;
 }
 
+#define CSI "\033["
+
+static bool SeqEnd(int ch) {
+	return (ch == 0) || ((ch >= '@') && (ch <= '~'));
+}
+
+static void RemoveEscSeq(std::string &s) {
+	size_t csi = s.find(CSI);
+	while (csi != std::string::npos) {
+		size_t endSeq = csi + 2;
+		while (endSeq < s.length() && !SeqEnd(s.at(endSeq)))
+			endSeq++;
+		s.erase(csi, endSeq-csi+1);
+		csi = s.find(CSI);
+	}
+}
+
 // Remove up to and including ch
 static void Chomp(std::string &s, int ch) {
 	const size_t posCh = s.find(static_cast<char>(ch));
@@ -1854,6 +1880,11 @@ void SciTEBase::ShowMessages(int line) {
 		std::string source;
 		int column;
 		int style = acc.StyleAt(startPosLine);
+		if ((style == SCE_ERR_ESCSEQ) || (style == SCE_ERR_ESCSEQ_UNKNOWN) || (style >= SCE_ERR_ES_BLACK)) {
+			// GCC message with ANSI escape sequences
+			RemoveEscSeq(message);
+			style = SCE_ERR_GCC;
+		}
 		int sourceLine = DecodeMessage(message.c_str(), source, style, column);
 		Chomp(message, ':');
 		if (style == SCE_ERR_GCC) {
@@ -1904,7 +1935,7 @@ void SciTEBase::GoMessage(int dir) {
 	Sci_CharacterRange crange;
 	crange.cpMin = wOutput.Call(SCI_GETSELECTIONSTART);
 	crange.cpMax = wOutput.Call(SCI_GETSELECTIONEND);
-	long selStart = crange.cpMin;
+	long selStart = static_cast<long>(crange.cpMin);
 	int curLine = wOutput.Call(SCI_LINEFROMPOSITION, selStart);
 	int maxLine = wOutput.Call(SCI_GETLINECOUNT);
 	int lookLine = curLine + dir;
@@ -1931,6 +1962,11 @@ void SciTEBase::GoMessage(int dir) {
 			wOutput.Call(SCI_MARKERADD, lookLine, 0);
 			wOutput.Call(SCI_SETSEL, startPosLine, startPosLine);
 			std::string message = GetRangeString(wOutput, startPosLine, startPosLine + lineLength);
+			if ((style == SCE_ERR_ESCSEQ) || (style == SCE_ERR_ESCSEQ_UNKNOWN) || (style >= SCE_ERR_ES_BLACK)) {
+				// GCC message with ANSI escape sequences
+				RemoveEscSeq(message);
+				style = SCE_ERR_GCC;
+			}
 			std::string source;
 			int column;
 			long sourceLine = DecodeMessage(message.c_str(), source, style, column);
@@ -1987,7 +2023,7 @@ void SciTEBase::GoMessage(int dir) {
 					const bool isAdd = message.find("+++ ") == 0;
 					const int atLine = lookLine + (isAdd ? 1 : 2); // lines are in this order: ---, +++, @@
 					std::string atMessage = GetLine(wOutput, atLine);
-					if (atMessage.find("@@ -") == 0) {
+					if (StartsWith(atMessage, "@@ -")) {
 						size_t atPos = 4; // deleted position starts right after "@@ -"
 						if (isAdd) {
 							const size_t linePlace = atMessage.find(" +", 7);
