@@ -6,6 +6,9 @@
 // The License.txt file describes the conditions under which this software may be distributed.
 
 #include <stdlib.h>
+// Older versions of GNU stdint.h require this definition to be able to see INT32_MAX
+#define __STDC_LIMIT_MACROS
+#include <stdint.h>
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
@@ -52,7 +55,7 @@ const GUI::gui_char propUserFileName[] = GUI_TEXT("SciTEUser.properties");
 const GUI::gui_char propGlobalFileName[] = GUI_TEXT("SciTEGlobal.properties");
 const GUI::gui_char propAbbrevFileName[] = GUI_TEXT("abbrev.properties");
 
-void SciTEBase::SetFileName(FilePath openName, bool fixCase) {
+void SciTEBase::SetFileName(const FilePath &openName, bool fixCase) {
 	if (openName.AsInternal()[0] == '\"') {
 		// openName is surrounded by double quotes
 		GUI::gui_string pathCopy = openName.AsInternal();
@@ -81,7 +84,7 @@ void SciTEBase::SetFileName(FilePath openName, bool fixCase) {
 	props.Set("FileNameExt", FileNameExt().AsUTF8().c_str());
 
 	SetWindowName();
-	if (buffers.buffers)
+	if (buffers.buffers.size() > 0)
 		buffers.buffers[buffers.Current()].Set(filePath);
 }
 
@@ -237,7 +240,7 @@ void SciTEBase::DiscoverIndentSetting() {
 	}
 }
 
-void SciTEBase::OpenCurrentFile(long fileSize, bool suppressMessage, bool asynchronous) {
+void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool asynchronous) {
 	if (CurrentBuffer()->pFileWorker) {
 		// Already performing an asynchronous load or save so do not restart load
 		if (!suppressMessage) {
@@ -269,19 +272,19 @@ void SciTEBase::OpenCurrentFile(long fileSize, bool suppressMessage, bool asynch
 		// Turn grey while loading
 		wEditor.Call(SCI_STYLESETBACK, STYLE_DEFAULT, 0xEEEEEE);
 		wEditor.Call(SCI_SETREADONLY, 1);
-		assert(CurrentBuffer()->pFileWorker == NULL);
+		assert(CurrentBufferConst()->pFileWorker == NULL);
 		ILoader *pdocLoad;
 		try {
-			pdocLoad = reinterpret_cast<ILoader *>(wEditor.CallReturnPointer(SCI_CREATELOADER, fileSize + 1000));
+			pdocLoad = reinterpret_cast<ILoader *>(wEditor.CallReturnPointer(SCI_CREATELOADER, static_cast<uptr_t>(fileSize) + 1000));
 		} catch (...) {
 			wEditor.Call(SCI_SETSTATUS, 0);
 			return;
 		}
-		CurrentBuffer()->pFileWorker = new FileLoader(this, pdocLoad, filePath, fileSize, fp);
+		CurrentBuffer()->pFileWorker = new FileLoader(this, pdocLoad, filePath, static_cast<size_t>(fileSize), fp);
 		CurrentBuffer()->pFileWorker->sleepTime = props.GetInt("asynchronous.sleep");
 		PerformOnNewThread(CurrentBuffer()->pFileWorker);
 	} else {
-		wEditor.Call(SCI_ALLOCATE, fileSize + 1000);
+		wEditor.Call(SCI_ALLOCATE, static_cast<uptr_t>(fileSize) + 1000);
 
 		Utf8_16_Read convert;
 		std::vector<char> data(blockSize);
@@ -448,8 +451,7 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 	}
 
 	if (errSaved) {
-		GUI::gui_string msg = LocaliseMessage("Could not save file '^0'.", pathSaved.AsInternal());
-		WindowMessageBox(wSciTE, msg);
+		FailedSaveMessageBox(pathSaved);
 	}
 
 	if (IsPropertiesFile(pathSaved)) {
@@ -491,7 +493,7 @@ bool SciTEBase::PreOpenCheck(const GUI::gui_char *) {
 	return false;
 }
 
-bool SciTEBase::Open(FilePath file, OpenFlags of) {
+bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	InitialiseBuffers();
 
 	FilePath absPath = file.AbsolutePath();
@@ -517,14 +519,22 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		return false;
 	}
 
-	const long fileSize = absPath.IsUntitled() ? 0 : absPath.GetFileLength();
+	const long long fileSize = absPath.IsUntitled() ? 0 : absPath.GetFileLength();
+	if (fileSize > INT32_MAX) {
+		const GUI::gui_string sSize = GUI::StringFromLongLong(fileSize);
+		const GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long, "
+			"larger than 2GB which is the largest SciTE can open.",
+			absPath.AsInternal(), sSize.c_str());
+		WindowMessageBox(wSciTE, msg, mbsIconWarning);
+		return false;
+	}
 	if (fileSize > 0) {
 		// Real file, not empty buffer
-		int maxSize = props.GetInt("max.file.size");
+		long long maxSize = props.GetLongLong("max.file.size", 2000000000LL);
 		if (maxSize > 0 && fileSize > maxSize) {
-			GUI::gui_string sSize = GUI::StringFromInteger(fileSize);
-			GUI::gui_string sMaxSize = GUI::StringFromInteger(maxSize);
-			GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long,\n"
+			const GUI::gui_string sSize = GUI::StringFromLongLong(fileSize);
+			const GUI::gui_string sMaxSize = GUI::StringFromLongLong(maxSize);
+			const GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long,\n"
 			        "larger than the ^2 bytes limit set in the properties.\n"
 			        "Do you still want to open it?",
 			        absPath.AsInternal(), sSize.c_str(), sMaxSize.c_str());
@@ -535,7 +545,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		}
 	}
 
-	if (buffers.size == buffers.length) {
+	if (buffers.size() == buffers.length) {
 		AddFileToStack(filePath, GetSelectedRange(), GetCurrentScrollPosition());
 		ClearDocument();
 		CurrentBuffer()->lifeState = Buffer::open;
@@ -547,7 +557,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		}
 	}
 
-	assert(CurrentBuffer()->pFileWorker == NULL);
+	assert(CurrentBufferConst()->pFileWorker == NULL);
 	SetFileName(absPath);
 
 	propsDiscovered.Clear();
@@ -723,9 +733,13 @@ bool SciTEBase::OpenSelected() {
 }
 
 void SciTEBase::Revert() {
-	RecentFile rf = GetFilePosition();
-	OpenCurrentFile(filePath.GetFileLength(), false, false);
-	DisplayAround(rf);
+	if (filePath.IsUntitled()) {
+		wEditor.Call(SCI_CLEARALL);
+	} else {
+		RecentFile rf = GetFilePosition();
+		OpenCurrentFile(filePath.GetFileLength(), false, false);
+		DisplayAround(rf);
+	}
 }
 
 void SciTEBase::CheckReload() {
@@ -854,7 +868,7 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureAll() {
 		wEditor.Call(SCI_SETDOCPOINTER, 0, buffers.buffers[0].doc);
 	}
 	// Release all the extra documents
-	for (int j = 0; j < buffers.size; j++) {
+	for (int j = 0; j < buffers.size(); j++) {
 		if (buffers.buffers[j].doc && !buffers.buffers[j].pFileWorker) {
 			wEditor.Call(SCI_RELEASEDOCUMENT, 0, buffers.buffers[j].doc);
 			buffers.buffers[j].doc = 0;
@@ -918,7 +932,7 @@ void SciTEBase::EnsureFinalNewLine() {
 }
 
 // Perform any changes needed before saving such as normalizing spaces and line ends.
-bool SciTEBase::PrepareBufferForSave(FilePath saveName) {
+bool SciTEBase::PrepareBufferForSave(const FilePath &saveName) {
 	bool retVal = false;
 	// Perform clean ups on text before saving
 	wEditor.Call(SCI_BEGINUNDOACTION);
@@ -944,14 +958,14 @@ bool SciTEBase::PrepareBufferForSave(FilePath saveName) {
 /**
  * Writes the buffer to the given filename.
  */
-bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
+bool SciTEBase::SaveBuffer(const FilePath &saveName, SaveFlags sf) {
 	bool retVal = PrepareBufferForSave(saveName);
 
 	if (!retVal) {
 
 		FILE *fp = saveName.Open(fileWrite);
 		if (fp) {
-			int lengthDoc = LengthDocument();
+			size_t lengthDoc = LengthDocument();
 			if (!(sf & sfSynchronous)) {
 				wEditor.Call(SCI_SETREADONLY, 1);
 				const char *documentBytes = reinterpret_cast<const char *>(wEditor.CallReturnPointer(SCI_GETCHARACTERPOINTER));
@@ -972,21 +986,23 @@ bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
 				convert.setfile(fp);
 				std::vector<char> data(blockSize + 1);
 				retVal = true;
-				int grabSize;
-				for (int i = 0; i < lengthDoc; i += grabSize) {
+				size_t grabSize;
+				for (size_t i = 0; i < lengthDoc; i += grabSize) {
 					grabSize = lengthDoc - i;
 					if (grabSize > blockSize)
 						grabSize = blockSize;
 					// Round down so only whole characters retrieved.
 					grabSize = wEditor.Call(SCI_POSITIONBEFORE, i + grabSize + 1) - i;
-					GetRange(wEditor, i, i + grabSize, &data[0]);
+					GetRange(wEditor, static_cast<int>(i), static_cast<int>(i + grabSize), &data[0]);
 					size_t written = convert.fwrite(&data[0], grabSize);
 					if (written == 0) {
 						retVal = false;
 						break;
 					}
 				}
-				convert.fclose();
+				if (convert.fclose() != 0) {
+					retVal = false;
+				}
 			}
 		}
 	}
@@ -1215,7 +1231,7 @@ class BufferedFile {
 		}
 	}
 public:
-	explicit BufferedFile(FilePath fPath) {
+	explicit BufferedFile(const FilePath &fPath) {
 		fp = fPath.Open(fileRead);
 		readAll = false;
 		exhausted = fp == NULL;
@@ -1256,11 +1272,11 @@ class FileReader {
 	std::string lineToCompare;
 	std::string lineToShow;
 	bool caseSensitive;
-	// Private so FileReader objects can not be copied
-	FileReader(const FileReader &);
+	// Deleted so FileReader objects can not be copied
+	FileReader(const FileReader &) = delete;
 public:
 
-	FileReader(FilePath fPath, bool caseSensitive_) {
+	FileReader(const FilePath &fPath, bool caseSensitive_) {
 		bf = new BufferedFile(fPath);
 		lineNum = 0;
 		lastWasCR = false;
@@ -1321,7 +1337,7 @@ bool SciTEBase::GrepIntoDirectory(const FilePath &directory) {
     return sDirectory[0] != '.';
 }
 
-void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *searchString, const GUI::gui_char *fileTypes) {
+void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char *searchString, const GUI::gui_char *fileTypes) {
 	FilePathSet directories;
 	FilePathSet files;
 	baseDir.List(directories, files);
@@ -1388,7 +1404,7 @@ void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const
 		os.append(GUI::UTF8FromString(fileTypes).c_str());
 		os.append("\"\n");
 		OutputAppendStringSynchronised(os.c_str());
-		MakeOutputVisible();
+		ShowOutputOnMainThread();
 		originalEnd += os.length();
 	}
 	std::string searchString(search);
