@@ -331,8 +331,8 @@ void SciTEBase::SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const
 }
 
 std::string SciTEBase::ExtensionFileName() const {
-	if (CurrentBuffer()->overrideExtension.length()) {
-		return CurrentBuffer()->overrideExtension;
+	if (CurrentBufferConst()->overrideExtension.length()) {
+		return CurrentBufferConst()->overrideExtension;
 	} else {
 		FilePath name = FileNameExt();
 		if (name.IsSet()) {
@@ -423,7 +423,9 @@ static const char *propertiesToForward[] = {
 	"fold.asm.explicit.start",
 	"fold.asm.syntax.based",
 	"fold.at.else",
+	"fold.baan.inner.level",
 	"fold.baan.keywords.based",
+	"fold.baan.sections",
 	"fold.baan.syntax.based",
 	"fold.basic.comment.explicit",
 	"fold.basic.explicit.anywhere",
