@@ -14,7 +14,7 @@ public:
 	int widthDesired;
 	int widthAllocated;
 	GUI::Window w;
-	UserControl(UCControlType controlType_, GUI::gui_string text_, int item_) : 
+	UserControl(UCControlType controlType_, const GUI::gui_string &text_, int item_) : 
 		controlType(controlType_),
 		text(text_),
 		item(item_),
@@ -39,7 +39,7 @@ public:
 	std::vector<std::vector<UserControl> > controls;
 	std::vector<ColumnWidth> widths;
 
-	StripDefinition(GUI::gui_string definition) {
+	explicit StripDefinition(GUI::gui_string definition) {
 		hasClose = false;
 		controls.clear();
 		columns = 0;
