@@ -32,10 +32,11 @@
 #define _WIN32_WINNT WIN_TARGET
 #define WINVER WIN_TARGET
 #else
-#define _WIN32_WINNT  0x0602
-#define WINVER 0x0602
+#define _WIN32_WINNT  0x0501
+#define WINVER 0x0501
 #endif
 #include <windows.h>
+#include <windowsx.h>
 #if defined(DISABLE_THEMES) || (defined(_MSC_VER) && (_MSC_VER <= 1200))
 // Old compilers do not have Uxtheme.h
 typedef void *HTHEME;
@@ -106,6 +107,7 @@ typedef void *HTHEME;
 const int SCITE_TRAY = WM_APP + 0;
 const int SCITE_DROP = WM_APP + 1;
 const int SCITE_WORKER = WM_APP + 2;
+const int SCITE_SHOWOUTPUT = WM_APP + 3;
 
 enum {
 	WORK_EXECUTE = WORK_PLATFORM + 1
@@ -228,12 +230,14 @@ protected:
 	virtual void GetWindowPosition(int *left, int *top, int *width, int *height, int *maximize);
 	void SetScaleFactor(int scale);
 
+	virtual void ReadEmbeddedProperties();
 	virtual void ReadPropertiesInitial();
 	virtual void ReadProperties();
 
 	virtual void TimerStart(int mask);
 	virtual void TimerEnd(int mask);
 
+	virtual void ShowOutputOnMainThread();
 	virtual void SizeContentWindows();
 	virtual void SizeSubWindows();
 
@@ -250,11 +254,12 @@ protected:
 	void LocaliseControl(HWND w);
 	void LocaliseDialog(HWND wDialog);
 
-	int DoDialog(HINSTANCE hInst, const TCHAR *resName, HWND hWnd, DLGPROC lpProc);
+	int DoDialog(const TCHAR *resName, DLGPROC lpProc);
+	HWND CreateParameterisedDialog(LPCWSTR lpTemplateName, DLGPROC lpProc);
 	GUI::gui_string DialogFilterFromProperty(const GUI::gui_char *filterProperty);
 	void CheckCommonDialogError();
-	virtual bool OpenDialog(FilePath directory, const GUI::gui_char *filesFilter);
-	FilePath ChooseSaveName(FilePath directory, const char *title, const GUI::gui_char *filesFilter = 0, const char *ext = 0);
+	virtual bool OpenDialog(const FilePath &directory, const GUI::gui_char *filesFilter);
+	FilePath ChooseSaveName(const FilePath &directory, const char *title, const GUI::gui_char *filesFilter = 0, const char *ext = 0);
 	virtual bool SaveAsDialog();
 	virtual void SaveACopy();
 	virtual void SaveAsHTML();
@@ -315,16 +320,16 @@ protected:
 	virtual void UserStripSetList(int control, const char *value);
 	virtual const char *UserStripValue(int control);
 	void UserStripClosed();
-	virtual void ShowBackgroundProgress(const GUI::gui_string &explanation, int size, int progress);
+	virtual void ShowBackgroundProgress(const GUI::gui_string &explanation, size_t size, size_t progress);
 	BOOL FindMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK FindDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK FindDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	BOOL ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK ReplaceDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK ReplaceDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	virtual void UIClosed();
 	void PerformGrep();
 	void FillCombos(Dialog &dlg);
 	BOOL GrepMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK GrepDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK GrepDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	virtual void FindIncrement();
 	virtual void Find();
 	virtual void FindInFiles();
@@ -333,25 +338,25 @@ protected:
 	virtual void DestroyFindReplace();
 
 	BOOL GoLineMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK GoLineDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK GoLineDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	virtual void GoLineDialog();
 
 	BOOL AbbrevMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK AbbrevDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK AbbrevDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	virtual bool AbbrevDialog();
 
 	BOOL TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK TabSizeDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK TabSizeDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	virtual void TabSizeDialog();
 
 	virtual bool ParametersOpen();
 	void ParamGrab();
 	virtual bool ParametersDialog(bool modal);
 	BOOL ParametersMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK ParametersDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK ParametersDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 
 	BOOL AboutMessage(HWND hDlg, UINT message, WPARAM wParam);
-	static BOOL CALLBACK AboutDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
+	static INT_PTR CALLBACK AboutDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	void AboutDialogWithBuild(int staticBuild_);
 
 	void RestorePosition();
@@ -368,7 +373,7 @@ public:
 	/// Management of the command line parameters.
 	void Run(const GUI::gui_char *cmdLine);
 	uptr_t EventLoop();
-	void OutputAppendEncodedStringSynchronised(GUI::gui_string s, int codePageDocument);
+	void OutputAppendEncodedStringSynchronised(const GUI::gui_string &s, int codePageDocument);
 	void ResetExecution();
 	void ExecuteNext();
 	DWORD ExecuteOne(const Job &jobToRun);
@@ -409,6 +414,8 @@ inline bool IsKeyDown(int key) {
 	return (::GetKeyState(key) & 0x80000000) != 0;
 }
 
+// Common minor conversions
+
 inline GUI::Point PointFromLong(LPARAM lPoint) {
 	return GUI::Point(static_cast<short>(LOWORD(lPoint)), static_cast<short>(HIWORD(lPoint)));
 }
@@ -416,3 +423,15 @@ inline GUI::Point PointFromLong(LPARAM lPoint) {
 inline int ControlIDOfWParam(WPARAM wParam) {
 	return wParam & 0xffff;
 }
+
+inline HWND HwndOf(GUI::Window w) {
+	return static_cast<HWND>(w.GetID());
+}
+
+inline HMENU HmenuID(size_t id) {
+	return reinterpret_cast<HMENU>(id);
+}
+
+inline POINT *PointPointer(GUI::Point *pt) {
+	return reinterpret_cast<POINT *>(pt);
+}
