@@ -17,12 +17,12 @@ void SciTEWin::SetFileProperties(
 	char temp[TEMP_LEN];
 	HANDLE hf = ::CreateFileW(filePath.AsInternal(), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 	if (hf != INVALID_HANDLE_VALUE) {
-		FILETIME ft = {};
+		FILETIME ft = FILETIME();
 		::GetFileTime(hf, NULL, NULL, &ft);
 		::CloseHandle(hf);
-		FILETIME lft = {};
+		FILETIME lft = FILETIME();
 		::FileTimeToLocalFileTime(&ft, &lft);
-		SYSTEMTIME st = {};
+		SYSTEMTIME st = SYSTEMTIME();
 		if (::FileTimeToSystemTime(&lft, &st) == 0)
 			st = SYSTEMTIME();
 		::GetTimeFormatA(LOCALE_USER_DEFAULT,
@@ -540,7 +540,7 @@ void SciTEWin::LocaliseMenu(HMENU hmenu) {
 	for (int i = 0; i <= ::GetMenuItemCount(hmenu); i++) {
 		GUI::gui_char buff[200];
 		buff[0] = '\0';
-		MENUITEMINFOW mii = {};
+		MENUITEMINFOW mii = MENUITEMINFOW();
 		mii.cbSize = sizeof(mii);
 		mii.fMask = MIIM_CHECKMARKS | MIIM_DATA | MIIM_ID |
 		            MIIM_STATE | MIIM_SUBMENU | MIIM_TYPE;
@@ -964,7 +964,7 @@ void SciTEWin::Creation() {
 
 	wTabBar.Show();
 
-	::CreateWindowEx(
+	HWND hwnd = ::CreateWindowEx(
 	               0,
 	               classNameInternal,
 	               TEXT("userStrip"),
@@ -975,8 +975,10 @@ void SciTEWin::Creation() {
 	               HmenuID(2001),
 	               hInstance,
 	               &userStrip);
+	if (!hwnd)
+		exit(FALSE);
 
-	::CreateWindowEx(
+	hwnd = ::CreateWindowEx(
 	               0,
 	               classNameInternal,
 	               TEXT("backgroundStrip"),
@@ -987,8 +989,10 @@ void SciTEWin::Creation() {
 	               HmenuID(2001),
 	               hInstance,
 	               &backgroundStrip);
+	if (!hwnd)
+		exit(FALSE);
 
-	::CreateWindowEx(
+	hwnd = ::CreateWindowEx(
 	               0,
 	               classNameInternal,
 	               TEXT("searchStrip"),
@@ -999,8 +1003,10 @@ void SciTEWin::Creation() {
 	               HmenuID(2001),
 	               hInstance,
 	               &searchStrip);
+	if (!hwnd)
+		exit(FALSE);
 
-	::CreateWindowEx(
+	hwnd = ::CreateWindowEx(
 	               0,
 	               classNameInternal,
 	               TEXT("FindStrip"),
@@ -1011,8 +1017,10 @@ void SciTEWin::Creation() {
 	               HmenuID(2002),
 	               hInstance,
 	               &findStrip);
+	if (!hwnd)
+		exit(FALSE);
 
-	::CreateWindowEx(
+	hwnd = ::CreateWindowEx(
 	               0,
 	               classNameInternal,
 	               TEXT("ReplaceStrip"),
@@ -1023,6 +1031,8 @@ void SciTEWin::Creation() {
 	               HmenuID(2003),
 	               hInstance,
 	               &replaceStrip);
+	if (!hwnd)
+		exit(FALSE);
 
 	wStatusBar = ::CreateWindowEx(
 	                 0,
