@@ -229,7 +229,7 @@ GUI::Window Strip::CreateButton(const char *text, size_t ident, bool check) {
 		}
 	}
 #endif
-	TOOLINFO toolInfo = {};
+	TOOLINFO toolInfo = TOOLINFO();
 	toolInfo.cbSize = sizeof(toolInfo);
 	toolInfo.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
 	toolInfo.hinst = ::GetModuleHandle(NULL);
@@ -526,7 +526,7 @@ LRESULT Strip::CustomDraw(NMHDR *pnmh) {
 			pnmh->hwndFrom, BM_GETIMAGE, IMAGE_BITMAP, 0));
 
 		// Retrieve the bitmap dimensions
-		BITMAPINFO rbmi = {};
+		BITMAPINFO rbmi = BITMAPINFO();
 		rbmi.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
 		::GetDIBits(pcd->hdc, hBitmap, 0, 0, NULL, &rbmi, DIB_RGB_COLORS);
 
@@ -749,12 +749,14 @@ LRESULT BackgroundStrip::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 	return 0l;
 }
 
-void BackgroundStrip::SetProgress(const GUI::gui_string &explanation, int size, int progress) {
+void BackgroundStrip::SetProgress(const GUI::gui_string &explanation, size_t size, size_t progress) {
 	if (explanation != ControlGText(wExplanation)) {
 		::SetWindowTextW(HwndOf(wExplanation), explanation.c_str());
 	}
-	::SendMessage(HwndOf(wProgress), PBM_SETRANGE32, 0, size);
-	::SendMessage(HwndOf(wProgress), PBM_SETPOS, progress, 0);
+	// Scale values by 1000 as PBM_SETRANGE32 limited to 32-bit
+	const int scaleProgress = 1000;
+	::SendMessage(HwndOf(wProgress), PBM_SETRANGE32, 0, size/scaleProgress);
+	::SendMessage(HwndOf(wProgress), PBM_SETPOS, progress/scaleProgress, 0);
 }
 
 static const COLORREF colourNoMatch = RGB(0xff,0x66,0x66);
